@@ -1,3 +1,4 @@
+
 /*---------------------------------------------------------------
  * Copyright (c) 1999,2000,2001,2002,2003
  * The Board of Trustees of the University of Illinois
@@ -81,7 +82,9 @@ extern "C" {
 #if HAVE_THREAD_DEBUG
 #include <time.h>
 #include <unistd.h>
+#if HAVE_GETTID_SYSCALL
 #include <sys/syscall.h>
+#endif
 #include <sys/types.h>
 #include <stdarg.h>
 #include "Reporter.h"
@@ -115,7 +118,11 @@ static void __gettimestamp(char *timestr) {
     struct tm *t;
     t=localtime(&t1.tv_sec);
     if (t) {
+#if WIN32
+	strftime(timestr, 200, "%Y-%m-%d %H:%M:%S", t);
+#else
         strftime(timestr, 200, "%T", t);
+#endif
         // strftime(buf, len, "%F %T", &t);
 	snprintf(&timestr[strlen(timestr)], strlen(timestr), ".%09ld", t1.tv_nsec);
         timestr[199]='\0';
@@ -210,9 +217,12 @@ void thread_start_all(struct thread_Settings* thread) {
 
 void thread_start(struct thread_Settings* thread) {
     // Make sure this object has not been started already
-    if (thread_equalid(thread->mTID, thread_zeroid())) {
-
-
+    if (!thread_equalid(thread->mTID, thread_zeroid())) {
+	WARN(1, "thread_start called on running thread");
+#if HAVE_THREAD_DEBUG
+	thread_debug("Thread_start info %p id=%d ", (void *)thread, (int)thread->mTID);
+#endif
+    } else {
         // increment thread count
         Condition_Lock(thread_sNum_cond);
         thread_sNum++;
@@ -222,7 +232,6 @@ void thread_start(struct thread_Settings* thread) {
         Condition_Unlock(thread_sNum_cond);
 
 #if defined(HAVE_POSIX_THREAD)
-
         // pthreads -- spawn new thread
         if (pthread_create(&thread->mTID, NULL, thread_run_wrapper, thread) != 0) {
             WARN(1, "pthread_create");
@@ -241,11 +250,9 @@ void thread_start(struct thread_Settings* thread) {
             Condition_Unlock(thread_sNum_cond);
         }
 #if HAVE_THREAD_DEBUG
-	thread_debug("Thread_run_wrapper(%p mode=%x) thread counts tot/trfc=%d/%d", (void *)thread, thread->mThreadMode, thread_sNum, thread_trfc_sNum);
+	thread_debug("Thread_run_wrapper(%p mode=%x) thread counts tot/trfc=%d/%d (id=%d)", (void *)thread, thread->mThreadMode, thread_sNum, thread_trfc_sNum, (int)thread->mTID);
 #endif
-
 #elif defined(HAVE_WIN32_THREAD)
-
         // Win32 threads -- spawn new thread
         // Win32 has a thread handle in addition to the thread ID
         thread->mHandle = CreateThread(NULL, 0, thread_run_wrapper, thread, 0, &thread->mTID);
@@ -256,13 +263,11 @@ void thread_start(struct thread_Settings* thread) {
             Condition_Lock(thread_sNum_cond);
             thread_sNum--;
 	    if ((thread->mThreadMode == kMode_Client) || (thread->mThreadMode == kMode_Server)) {
-	      thread_trfc_sNum--;
+		thread_trfc_sNum--;
 	    }
             Condition_Unlock(thread_sNum_cond);
         }
-
 #else
-
         // single-threaded -- call Run_Wrapper in this thread
         thread_run_wrapper(thread);
 #endif
@@ -377,7 +382,7 @@ thread_run_wrapper(void* paramPtr) {
 
 #ifdef HAVE_POSIX_THREAD
     // detach Thread. If someone already joined it will not do anything
-    // If noone has then it will free resources upon return from this
+    // If none has then it will free resources upon return from this
     // function (Run_Wrapper)
     pthread_detach(thread->mTID);
 #endif
