@@ -89,15 +89,16 @@ public:
     int StartSynch(void);
     void TxDelay(void);
     void ConnectPeriodic(void);
-    void my_connect(void);
-    bool isConnected(void);
-    void SendFirstPayload(void);
+    bool my_connect(bool close_on_fail);
+    bool isConnected(void) const;
+    int SendFirstPayload(void);
     int BarrierClient(struct BarrierMutex *);
     struct ReportHeader *myJob;
 
 private:
     inline void WritePacketID(intmax_t);
     inline void WriteTcpTxHdr(struct ReportStruct *, int, int);
+    inline double get_delay_target(void);
     void InitTrafficLoop(void);
     void SetReportStartTime(void);
     inline void SetFullDuplexReportStartTime(void);
@@ -112,19 +113,24 @@ private:
     double delay_lower_bounds;
     intmax_t totLen;
     bool one_report;
+    bool apply_first_udppkt_delay;
     int udp_payload_minimum;
-
+    void myReportPacket (void);
+#ifdef HAVE_STRUCT_TCP_INFO_TCPI_TOTAL_RETRANS
+    struct tcp_info my_tcpi_stats;
+    bool myReportPacket (bool sample_tcpi);
+#endif
     // TCP plain
     void RunTCP(void);
     // TCP version which supports rate limiting per -b
     void RunRateLimitedTCP(void);
+    void RunNearCongestionTCP(void);
     // UDP traffic with isochronous and vbr support
     void RunUDPIsochronous(void);
     // UDP plain
     void RunUDP(void);
     // client connect
     void PeerXchange(void);
-
     thread_Settings *mSettings;
 #if WIN32
     SOCKET mySocket;
@@ -139,6 +145,8 @@ private:
     char* readAt;
     Timestamp connect_done, connect_start;
     Isochronous::FrameCounter *framecounter;
+    bool isburst;
+    bool peerclose;
 }; // end class Client
 
 #endif // CLIENT_H
