@@ -72,6 +72,7 @@ public:
 
 private:
     int mClients;
+    int mBufLen;
     char* mBuf; // used for UDP packet or TCP  messages
     struct ether_header *eth_hdr;
     struct iphdr *ip_hdr;
@@ -79,6 +80,8 @@ private:
     thread_Settings *mSettings;
     thread_Settings *server;
     Timestamp mEndTime;
+    bool apply_client_settings_udp(thread_Settings *server);
+    bool apply_client_settings_tcp(thread_Settings *server);
     bool apply_client_settings(thread_Settings *server);
     int client_test_ack(thread_Settings *server);
     void my_multicast_join(void);
@@ -87,6 +90,7 @@ private:
     int udp_accept(thread_Settings *server);
     bool L2_setup(thread_Settings *server, int sockfd);
     void UDPSingleServer(thread_Settings *server);
+    bool test_permit_key(uint32_t flags, thread_Settings *server, int keyoffset);
 #if WIN32
     SOCKET ListenSocket;
 #else
