@@ -61,6 +61,7 @@
 // forward declarations found in Settings.hpp
 struct thread_Settings;
 struct server_hdr;
+
 #include "Settings.hpp"
 
 #define NUM_REPORT_STRUCTS 10000
@@ -68,11 +69,11 @@ struct server_hdr;
 #define NETPOWERCONSTANT 1e-6
 #define REPORTTXTMAX 80
 #define MINBARRIERTIMEOUT 3
-
+#define PARTIALPERCENT 0.25 // used to decide if a final partial report should be displayed
 // If the minimum latency exceeds the boundaries below
 // assume the clocks are not synched and suppress the
 // latency output. Units are seconds
-#define UNREALISTIC_LATENCYMINMIN -1
+#define UNREALISTIC_LATENCYMINMIN -0.01
 #define UNREALISTIC_LATENCYMINMAX 60
 
 #ifdef __cplusplus
@@ -81,6 +82,8 @@ extern "C" {
 
 extern struct Condition ReportCond;
 extern struct Condition ReportsPending;
+extern int groupID;
+extern Mutex transferid_mutex;
 
 /*
  *
@@ -140,19 +143,6 @@ struct WriteStats {
     int up_to_date;
 };
 
-struct IsochStats {
-    int mFPS; //frames per second
-    double mMean; //variable bit rate mean
-    double mVariance; //vbr variance
-    int mJitterBufSize; //Server jitter buffer size, units is frames
-    intmax_t slipcnt;
-    intmax_t framecnt;
-    intmax_t framelostcnt;
-    unsigned int mBurstInterval;
-    unsigned int mBurstIPG; //IPG of packets within the burst
-    uint32_t frameID;
-};
-
 /*
  * This struct contains all important information from the sending or
  * recieving thread.
@@ -207,14 +197,19 @@ union SendReadStats {
 // rather than using references
 struct ReportCommon {
     enum ThreadMode ThreadMode;
+    enum ReportMode ReportMode;
+    bool KeyCheck;
     int flags;
     int flags_extend;
+    int flags_extend2;
     int threads;
     unsigned short Port;
+    unsigned short PortLast;
     unsigned short BindPort;
     unsigned short ListenPort;
-    intmax_t UDPRate;            // -b or -u
-    int UDPRateUnits;
+    intmax_t AppRate;            // -b or -u
+    uint32_t BurstSize;
+    int AppRateUnits;
     char Format;
     int TTL;
     int BufLen;
@@ -236,6 +231,12 @@ struct ReportCommon {
     char* Ifrnametx;
     char* SSMMulticastStr;
     char* Congestion;
+    char* transferIDStr;
+    char* PermitKey;
+    int transferID;
+    double rtt_weight;
+    double ListenerTimeout;
+    double FPS;
 #if WIN32
     SOCKET socket;
 #else
@@ -248,8 +249,9 @@ struct ReportCommon {
 
 struct ConnectionInfo {
     struct ReportCommon *common;
+    struct timeval connect_timestamp;
     double connecttime;
-    double txholdbacktime;
+    struct timeval txholdbacktime;
     struct timeval epochStartTime;
     int winsize;
     char peerversion[PEERVERBUFSIZE];
@@ -257,28 +259,6 @@ struct ConnectionInfo {
     int MSS;
 };
 
-struct ReportSettings {
-    struct ReportCommon *common;
-    iperf_sockaddr peer;
-    Socklen_t size_peer;
-    iperf_sockaddr local;
-    Socklen_t size_local;
-    intmax_t UDPRate;            // -b or -u
-    int UDPRateUnits;            // -b is either bw or pps
-    int pid;
-    struct IsochStats isochstats;
-    void (*output_handler) (struct ReportSettings *settings);
-};
-
-// Timestamps
-enum TimeStampType {
-    INTERVAL  = 0,
-    FINALPARTIAL,
-    TOTAL,
-    FRAME
-};
-
-
 struct ShiftIntCounter {
     intmax_t current;
     intmax_t prev;
@@ -297,13 +277,50 @@ struct ShiftCounters {
     struct ShiftIntCounter IPG;
 };
 
+struct IsochStats {
+    double mFPS; //frames per second
+    double mMean; //variable bit rate mean
+    double mVariance; //vbr variance
+    int mJitterBufSize; //Server jitter buffer size, units is frames
+    uintmax_t cntFrames;
+    uintmax_t cntFramesMissed;
+    uintmax_t cntSlips;
+    struct ShiftUintCounter slipcnt;
+    struct ShiftUintCounter framecnt;
+    struct ShiftUintCounter framelostcnt;
+    unsigned int mBurstInterval;
+    unsigned int mBurstIPG; //IPG of packets within the burst
+    uint32_t frameID;
+};
+
+struct ReportSettings {
+    struct ReportCommon *common;
+    iperf_sockaddr peer;
+    Socklen_t size_peer;
+    iperf_sockaddr local;
+    Socklen_t size_local;
+    int pid;
+    struct IsochStats isochstats;
+    void (*output_handler) (struct ReportSettings *settings);
+};
+
+// Timestamps
+enum TimeStampType {
+    INTERVAL  = 0,
+    FINALPARTIAL,
+    TOTAL,
+    FRAME
+};
+
 struct ReportTimeStamps {
     double iStart;
     double iEnd;
+    double significant_partial;
     struct timeval startTime;
     struct timeval matchTime;
     struct timeval packetTime;
     struct timeval prevpacketTime;
+    struct timeval prevsendTime;
     struct timeval nextTime;
     struct timeval intervalTime;
     struct timeval IPGstart;
@@ -313,10 +330,9 @@ struct TransferInfo {
     struct ReportCommon *common;
     struct ReportTimeStamps ts;
     void (*output_handler) (struct TransferInfo *stats);
-    int transferID;
     int groupID;
     int threadcnt;
-    int sumflag;
+    int filter_this_sample_output;
     uintmax_t cntBytes;
     intmax_t cntError;
     intmax_t cntOutofOrder;
@@ -338,6 +354,8 @@ struct TransferInfo {
     uint32_t matchframeID;
     uint32_t frameID;
     char csv_peer[CSVPEERLIMIT];
+    bool final;
+    bool burstid_transition;
 };
 
 struct SumReport {
@@ -346,6 +364,7 @@ struct SumReport {
     struct TransferInfo info;
     void (*transfer_protocol_sum_handler) (struct TransferInfo *stats, int final);
     struct BarrierMutex fullduplex_barrier;
+    int sum_fd_set;
 };
 
 struct ReporterData {
@@ -353,6 +372,7 @@ struct ReporterData {
     void (*packet_handler) (struct ReporterData *data, struct ReportStruct *packet);
     void (*transfer_protocol_handler) (struct ReporterData *data, int final);
     int (*transfer_interval_handler) (struct ReporterData *data, struct ReportStruct *packet);
+    bool burst_boundary;
 
     struct PacketRing *packetring;
     int reporter_thread_suspends; // used to detect CPU bound systems
@@ -360,7 +380,6 @@ struct ReporterData {
     // group sum and full duplext reports
     struct SumReport *GroupSumReport;
     struct SumReport *FullDuplexReport;
-
     struct TransferInfo info;
 };
 
@@ -384,15 +403,19 @@ typedef void (* report_settings)( struct ReporterData* );
 typedef void (* report_statistics)( struct TransferInfo* );
 typedef void (* report_serverstatistics)( struct ConnectionInfo *, struct TransferInfo* );
 
-void SetSumHandlers (struct thread_Settings *inSettings, struct SumReport* sumreport, int fullduplex);
+void SetSumHandlers (struct thread_Settings *inSettings, struct SumReport* sumreport);
 struct SumReport* InitSumReport(struct thread_Settings *inSettings, int inID, int fullduplex);
 struct ReportHeader* InitIndividualReport(struct thread_Settings *inSettings);
 struct ReportHeader* InitConnectionReport(struct thread_Settings *inSettings, double ct);
-struct ConnectionInfo* InitConnectOnlyReport(struct thread_Settings *inSettings);
+struct ConnectionInfo* InitConnectOnlyReport(struct thread_Settings *thread);
 struct ReportHeader *InitSettingsReport(struct thread_Settings *inSettings);
 struct ReportHeader* InitServerRelayUDPReport(struct thread_Settings *inSettings, struct server_hdr *server);
 void PostReport(struct ReportHeader *reporthdr);
-void ReportPacket(struct ReporterData *reporthdr, struct ReportStruct *packet);
+#ifdef HAVE_STRUCT_TCP_INFO_TCPI_TOTAL_RETRANS
+bool ReportPacket (struct ReporterData* data, struct ReportStruct *packet, struct tcp_info *tcp_stats);
+#else
+void ReportPacket (struct ReporterData* data, struct ReportStruct *packet);
+#endif
 int EndJob(struct ReportHeader *reporthdr,  struct ReportStruct *packet);
 void FreeReport(struct ReportHeader *reporthdr);
 void FreeSumReport (struct SumReport *sumreport);
@@ -400,8 +423,8 @@ void FreeConnectionReport(struct ConnectionInfo *reporthdr);
 void ReportServerUDP(struct thread_Settings *inSettings, struct server_hdr *server);
 void ReportConnections(struct thread_Settings *inSettings );
 void reporter_dump_job_queue(void);
-void IncrSumReportRefCounter(struct SumReport *multihdr);
-int DecrSumReportRefCounter(struct SumReport *multihdr);
+void IncrSumReportRefCounter(struct SumReport *sumreport);
+int DecrSumReportRefCounter(struct SumReport *sumreport);
 
 extern struct AwaitMutex reporter_state;
 extern struct AwaitMutex threads_start;
@@ -415,28 +438,34 @@ extern report_statistics multiple_reports[];
 
 // Packet accounting routines
 void reporter_handle_packet_null(struct ReporterData *report, struct ReportStruct *packet);
-void reporter_handle_packet_server_udp(struct ReporterData *report, struct ReportStruct *packet);
-void reporter_handle_packet_server_tcp(struct ReporterData *report, struct ReportStruct *packet);
-void reporter_handle_packet_client(struct ReporterData *report, struct ReportStruct *packet);
+void reporter_handle_packet_server_udp(struct ReporterData *data, struct ReportStruct *packet);
+void reporter_handle_packet_server_tcp(struct ReporterData *data, struct ReportStruct *packet);
+void reporter_handle_packet_client(struct ReporterData *data, struct ReportStruct *packet);
 void reporter_handle_packet_pps(struct ReporterData *data, struct ReportStruct *packet);
 void reporter_handle_packet_isochronous(struct ReporterData *data, struct ReportStruct *packet);
 
 // Reporter's conditional prints, right now have time and frame based sampling, possibly add packet based
-int reporter_condprint_time_interval_report(struct ReporterData *reporthdr, struct ReportStruct *packet);
-int reporter_condprint_frame_interval_report_udp(struct ReporterData *reporthdr, struct ReportStruct *packet);
-int reporter_condprint_frame_interval_report_tcp(struct ReporterData *reporthdr, struct ReportStruct *packet);
+int reporter_condprint_time_interval_report(struct ReporterData *data, struct ReportStruct *packet);
+int reporter_condprint_frame_interval_report_client_udp(struct ReporterData *reporthdr, struct ReportStruct *packet);
+int reporter_condprint_frame_interval_report_server_udp(struct ReporterData *data, struct ReportStruct *packet);
+int reporter_condprint_frame_interval_report_server_tcp(struct ReporterData *data, struct ReportStruct *packet);
+int reporter_condprint_frame_interval_report_client_tcp(struct ReporterData *reporthdr, struct ReportStruct *packet);
+int reporter_condprint_burst_interval_report_client_udp(struct ReporterData *reporthdr, struct ReportStruct *packet);
+int reporter_condprint_burst_interval_report_server_udp(struct ReporterData *data, struct ReportStruct *packet);
+int reporter_condprint_burst_interval_report_server_tcp(struct ReporterData *data, struct ReportStruct *packet);
+int reporter_condprint_burst_interval_report_client_tcp(struct ReporterData *reporthdr, struct ReportStruct *packet);
 //void reporter_set_timestamps_time(struct ReporterData *stats, enum TimestampType);
 
-// Reporter's interval ouput specialize routines
+// Reporter's interval output specialize routines
 void reporter_transfer_protocol_null(struct ReporterData *stats, int final);
 //void reporter_transfer_protocol_reports(struct ReporterData *stats, struct ReportStruct *packet);
 //void reporter_transfer_protocol_multireports(struct ReporterData *stats, struct ReportStruct *packet);
-void reporter_transfer_protocol_client_tcp(struct ReporterData *stats, int final);
-void reporter_transfer_protocol_client_udp(struct ReporterData *stats, int final);
-void reporter_transfer_protocol_server_tcp(struct ReporterData *stats, int final);
-void reporter_transfer_protocol_server_udp(struct ReporterData *stats, int final);
+void reporter_transfer_protocol_client_tcp(struct ReporterData *data, int final);
+void reporter_transfer_protocol_client_udp(struct ReporterData *data, int final);
+void reporter_transfer_protocol_server_tcp(struct ReporterData *data, int final);
+void reporter_transfer_protocol_server_udp(struct ReporterData *data, int final);
 
-// Reporter's sum ouput routines (per -P > 1)
+// Reporter's sum output routines (per -P > 1)
 void reporter_transfer_protocol_sum_client_tcp(struct TransferInfo *stats, int final);
 void reporter_transfer_protocol_sum_server_tcp(struct TransferInfo *stats, int final);
 void reporter_transfer_protocol_sum_client_udp(struct TransferInfo *stats, int final);
@@ -454,20 +483,22 @@ void tcp_output_sum_read(struct TransferInfo *stats);
 void tcp_output_sum_read_enhanced(struct TransferInfo *stats);
 void tcp_output_sumcnt_read(struct TransferInfo *stats);
 void tcp_output_sumcnt_read_enhanced (struct TransferInfo *stats);
+void tcp_output_frame_read(struct TransferInfo *stats);
+void tcp_output_frame_read_triptime(struct TransferInfo *stats);
+void tcp_output_burst_read(struct TransferInfo *stats);
+
 // TCP client
 void tcp_output_write(struct TransferInfo *stats);
 void tcp_output_sum_write(struct TransferInfo *stats);
 void tcp_output_sumcnt_write(struct TransferInfo *stats);
 void tcp_output_write_enhanced (struct TransferInfo *stats);
+void tcp_output_write_enhanced_isoch (struct TransferInfo *stats);
 void tcp_output_sum_write_enhanced (struct TransferInfo *stats);
 void tcp_output_sumcnt_write_enhanced (struct TransferInfo *stats);
 // TCP fullduplex
-void tcp_output_fullduplex_write(struct TransferInfo *stats);
-void tcp_output_fullduplex_write_enhanced(struct TransferInfo *stats);
-void tcp_output_fullduplex_read(struct TransferInfo *stats);
-void tcp_output_fullduplex_read_enhanced(struct TransferInfo *stats);
 void tcp_output_fullduplex(struct TransferInfo *stats);
 void tcp_output_fullduplex_enhanced(struct TransferInfo *stats);
+void tcp_output_fullduplex_sum (struct TransferInfo *stats);
 
 // UDP server
 void udp_output_read(struct TransferInfo *stats);
@@ -475,7 +506,8 @@ void udp_output_read_enhanced(struct TransferInfo *stats);
 void udp_output_read_enhanced_triptime(struct TransferInfo *stats);
 void udp_output_read_enhanced_triptime_isoch(struct TransferInfo *stats);
 void udp_output_sum_read(struct TransferInfo *stats);
-void udp_output_sumcnt_read(struct TransferInfo *stats);
+void udp_output_sum_read_enhanced (struct TransferInfo *stats);
+void udp_output_sumcnt(struct TransferInfo *stats);
 void udp_output_sumcnt_read_enhanced (struct TransferInfo *stats);
 
 //UDP client
@@ -486,10 +518,12 @@ void udp_output_write_enhanced_isoch(struct TransferInfo *stats);
 void udp_output_sum_write_enhanced (struct TransferInfo *stats);
 void udp_output_sumcnt_write(struct TransferInfo *stats);
 void udp_output_sumcnt_write_enhanced (struct TransferInfo *stats);
+void udp_output_sumcnt_enhanced(struct TransferInfo *stats);
 // UDP full duplex
 void udp_output_fullduplex(struct TransferInfo *stats);
-void udp_output_fullduplex_sum(struct TransferInfo *stats);
 void udp_output_fullduplex_enhanced(struct TransferInfo *stats);
+void udp_output_fullduplex_sum(struct TransferInfo *stats);
+
 
 // CSV output
 void udp_output_basic_csv(struct TransferInfo *stats);
@@ -509,6 +543,12 @@ void write_UDP_AckFIN(struct TransferInfo *stats);
 int reporter_process_transfer_report (struct ReporterData *this_ireport);
 int reporter_process_report (struct ReportHeader *reporthdr);
 
+void setTransferID(struct thread_Settings *inSettings, int role_reversal);
+
+#ifdef HAVE_STRUCT_TCP_INFO_TCPI_TOTAL_RETRANS
+void gettcpistats(struct ReporterData *data, int final, struct tcp_info *tcp_stats);
+#endif
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif
