@@ -83,6 +83,7 @@ public:
 private:
     thread_Settings *mSettings;
     char* mBuf;
+    int mBufLen;
     Timestamp mEndTime;
     Timestamp now;
     ReportStruct scratchpad;
@@ -101,6 +102,7 @@ private:
     int SkipFirstPayload(void);
     Timestamp connect_done;
     bool peerclose;
+    bool isburst;
 #if WIN32
     SOCKET mySocket;
     SOCKET myDropSocket;
