@@ -69,14 +69,16 @@ Usage: iperf [-s|-c host] [options]\n\
        iperf [-h|--help] [-v|--version]\n\
 \n\
 Client/Server:\n\
-  -b, --bandwidth #[kmgKMG | pps]  bandwidth to send at in bits/sec or packets per second\n\
-  -e, --enhancedreports    use enhanced reporting giving more tcp/udp and traffic information\n\
+  -b, --bandwidth #[kmgKMG | pps]  bandwidth to read/send at in bits/sec or packets/sec\n\
+  -e, --enhanced    use enhanced reporting giving more tcp/udp and traffic information\n\
   -f, --format    [kmgKMG]   format to report: Kbits, Mbits, KBytes, MBytes\n\
   -i, --interval  #        seconds between periodic bandwidth reports\n\
   -l, --len       #[kmKM]    length of buffer in bytes to read or write (Defaults: TCP=128K, v4 UDP=1470, v6 UDP=1450)\n\
   -m, --print_mss          print TCP maximum segment size (MTU - TCP/IP header)\n\
   -o, --output    <filename> output the report or error message to this specified file\n\
-  -p, --port      #        server port to listen on/connect to\n\
+  -p, --port      #        client/server port to listen/send on and to connect\n\
+      --permit-key         permit key to be used to verify client and server (TCP only)\n\
+      --sum-only           output sum only reports\n\
   -u, --udp                use UDP rather than TCP\n\
   -w, --window    #[KM]    TCP window size (socket buffer size)\n"
 #ifdef HAVE_SCHED_SETSCHEDULER
@@ -87,14 +89,19 @@ Client/Server:\n\
   -M, --mss       #        set TCP maximum segment size (MTU - 40 bytes)\n\
   -N, --nodelay            set TCP no delay, disabling Nagle's Algorithm\n\
   -S, --tos       #        set the socket's IP_TOS (byte) field\n\
+  -Z, --tcp-congestion <algo>  set TCP congestion control algorithm (Linux only)\n\
 \n\
 Server specific:\n\
+  -p, --port      #[-#]    server port(s) to listen on/connect to\n\
   -s, --server             run in server mode\n\
+  -1, --singleclient       run one server at a time\n\
+      --histograms         enable latency histograms\n\
+      --permit-key-timeout set the timeout for a permit key in seconds\n\
   -t, --time      #        time in seconds to listen for new connections as well as to receive traffic (default not set)\n\
       --udp-histogram #,#  enable UDP latency histogram(s) with bin width and count, e.g. 1,1000=1(ms),1000(bins)\n\
   -B, --bind <ip>[%<dev>]  bind to multicast address and optional device\n\
-  -H, --ssm-host <ip>      set the SSM source, use with -B for (S,G) \n\
   -U, --single_udp         run in single threaded UDP mode\n\
+      --sum-dstip          sum traffic threads based upon destination ip address (default is src ip)\n\
   -D, --daemon             run the server as a daemon\n"
 #ifdef WIN32
 "  -R, --remove             remove service in win32\n"
@@ -106,16 +113,29 @@ const char usage_long2[] = "\
 \n\
 Client specific:\n\
   -c, --client    <host>   run in client mode, connecting to <host>\n\
+      --connect-only       run a connect only test\n\
+      --connect-retries #  number of times to retry tcp connect\n\
   -d, --dualtest           Do a bidirectional test simultaneously (multiple sockets)\n\
-      --fullduplex         run fullduplexectional test over same socket (full duplex mode)\n\
+      --fq-rate #[kmgKMG]  bandwidth to socket pacing\n\
+      --full-duplex        run full duplex test using same socket\n\
       --ipg                set the the interpacket gap (milliseconds) for packets within an isochronous frame\n\
       --isochronous <frames-per-second>:<mean>,<stddev> send traffic in bursts (frames - emulate video traffic)\n\
       --incr-dstip         Increment the destination ip with parallel (-P) traffic threads\n\
+      --incr-dstport       Increment the destination port with parallel (-P) traffic threads\n\
+      --incr-srcip         Increment the source ip with parallel (-P) traffic threads\n\
+      --local-only         Set don't route on socket\n\
+      --near-congestion=[w] Use a weighted write delay per the sampled TCP RTT (experimental)\n\
+      --no-connect-sync    No sychronization after connect when -P or parallel traffic threads\n\
+      --no-udp-fin         No final server to client stats at end of UDP test\n\
   -n, --num       #[kmgKMG]    number of bytes to transmit (instead of -t)\n\
   -r, --tradeoff           Do a fullduplexectional test individually\n\
   -t, --time      #        time in seconds to transmit for (default 10 secs)\n\
+      --trip-times         enable end to end measurements (requires client and server clock sync)\n\
+      --txdelay-time       time in seconds to hold back after connect and before first write\n\
+      --txstart-time       unix epoch time to schedule first write and start traffic\n\
   -B, --bind [<ip> | <ip:port>] bind ip (and optional port) from which to source traffic\n\
   -F, --fileinput <name>   input the data to be transmitted from a file\n\
+  -H, --ssm-host <ip>      set the SSM source, use with -B for (S,G) \n\
   -I, --stdin              input the data to be transmitted from stdin\n\
   -L, --listenport #       port to receive fullduplexectional tests back on\n\
   -P, --parallel  #        number of parallel client threads to run\n"
@@ -125,10 +145,10 @@ Client specific:\n\
 "  -R                       Remove the windows service\n"
 "      --reverse            reverse the test (client receives, server sends)\n"
 #endif
-"  -T, --ttl       #        time-to-live, for multicast (default 1)\n\
+"  -S, --tos                IP DSCP or tos settings\n\
+  -T, --ttl       #        time-to-live, for multicast (default 1)\n\
   -V, --ipv6_domain        Set the domain to IPv6 (send packets over IPv6)\n\
   -X, --peer-detect        perform server version detection and version exchange\n\
-  -Z, --linux-congestion <algo>  set TCP congestion control algorithm (Linux only)\n\
 \n\
 Miscellaneous:\n\
   -x, --reportexclude [CDMSV]   exclude C(connection) D(data) M(multicast) S(settings) V(server) reports\n\
@@ -175,6 +195,9 @@ const char client_port[] =
 const char server_pid_port[] =
 "Server listening on %s port %d with pid %d\n";
 
+const char server_pid_portrange[] =
+"Server listening on %s ports %d-%d with pid %d\n";
+
 const char client_pid_port[] =
 "Client connecting to %s, %s port %d with pid %d (%d flows)\n";
 
@@ -203,13 +226,10 @@ const char join_multicast_sg_dev[] =
 "Joining multicast (S,G)=%s,%s w/iface %s\n";
 
 const char client_datagram_size[] =
-"Sending %d byte datagrams, IPG target: %4.2f us\n";
-
-const char client_datagram_size_tx_sync[] =
-"Sending %d byte datagrams, tx-sync periodic: %4.2f ms\n";
+"Sending %d byte datagrams, IPG target: %.2f us\n";
 
 const char client_datagram_size_kalman[] =
-"Sending %d byte datagrams, IPG target: %4.2f us (kalman adjust)\n";
+"Sending %d byte datagrams, IPG target: %.2f us (kalman adjust)\n";
 
 const char server_datagram_size[] =
 "Receiving %d byte datagrams\n";
@@ -227,7 +247,13 @@ const char wait_server_threads[] =
 "Waiting for server threads to complete. Interrupt again to force quit.\n";
 
 const char client_isochronous[] =
-"Isochronous: %d frames/sec mean=%s/s, stddev=%s/s, Period/IPG=%0.2f/%0.3f ms\n";
+"Isochronous: %0.2f frames/sec mean=%s/s, stddev=%s/s, Period/IPG=%0.2f/%.3f ms\n";
+
+const char client_burstperiod[] =
+"Bursting: %s every %0.2f seconds\n";
+
+const char server_burstperiod[] =
+"Burst wait timeout set to (2 * %0.2f) seconds (use --burst-period=<n secs> to change)\n";
 
 const char client_fq_pacing [] =
 "fair-queue socket pacing set to %s/s\n";
@@ -240,7 +266,7 @@ const char report_bw_header[] =
 "[ ID] Interval       Transfer     Bandwidth\n";
 
 const char report_bw_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec\n";
 
 const char report_sum_bw_format[] =
 "[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec\n";
@@ -250,26 +276,27 @@ const char report_sumcnt_bw_format[] =
 
 const char report_bw_jitter_loss_header[] =
 "[ ID] Interval       Transfer     Bandwidth        Jitter   Lost/Total Datagrams\n";
+
 const char report_bw_jitter_loss_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%)\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %" PRIdMAX "/%" PRIdMAX " (%.2g%%)\n";
 
 const char report_sum_bw_jitter_loss_format[] =
-"[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%)\n";
+"[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec   %" PRIdMAX "/%" PRIdMAX " (%.2g%%)\n";
+
+const char report_sumcnt_bw_jitter_loss_header[] =
+"[SUM-cnt] Interval     Transfer     Bandwidth       Lost/Total    PPS\n";
 
 const char report_sumcnt_bw_jitter_loss_format[] =
-"[SUM-%d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%)\n";
+"[SUM-%d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %" PRIdMAX "/%" PRIdMAX " %8.0f pps\n";
 
 /* -------------------------------------------------------------------
  * Enhanced reports (per -e)
  * ------------------------------------------------------------------- */
-const char report_bw_sumcnt_header[] =
+const char report_sumcnt_bw_header[] =
 "[SUM-cnt] Interval       Transfer     Bandwidth\n";
 
-const char client_report_epoch_start[] =
-"[%3d] Client traffic to start at %s (%ld.%ld in epoch/unix format)\n";
-
 const char client_report_epoch_start_current[] =
-"[%3d] Client traffic to start at %s (%ld.%ld) current time %s\n";
+"[%3d] Client traffic to start in %d seconds at %s current time is %s\n";
 
 const char client_write_size[] =
 "Write buffer size";
@@ -278,7 +305,7 @@ const char server_read_size[] =
 "Read buffer size";
 
 const char report_bw_enhanced_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec\n";
 
 const char report_sum_bw_enhanced_format[] =
 "[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec\n";
@@ -287,65 +314,89 @@ const char report_bw_read_enhanced_header[] =
 "[ ID] Interval" IPERFTimeSpace "Transfer    Bandwidth       Reads=Dist\n";
 
 const char report_bw_read_enhanced_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d=%d:%d:%d:%d:%d:%d:%d:%d\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %d=%d:%d:%d:%d:%d:%d:%d:%d\n";
+
+const char report_sumcnt_bw_read_enhanced_header[] =
+"[SUM-cnt] Interval" IPERFTimeSpace "Transfer    Bandwidth       Reads=Dist\n";
+
+const char report_sumcnt_bw_read_enhanced_format[] =
+"[SUM-%d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d=%d:%d:%d:%d:%d:%d:%d:%d\n";
 
 const char report_bw_read_enhanced_netpwr_header[] =
 "[ ID] Interval" IPERFTimeSpace "Transfer    Bandwidth    Burst Latency avg/min/max/stdev (cnt/size) inP NetPwr  Reads=Dist\n";
 
 const char report_bw_read_enhanced_netpwr_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f/%6.3f/%6.3f/%6.3f ms (%d/%d) %s %4.2f  %d=%d:%d:%d:%d:%d:%d:%d:%d\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %.3f/%.3f/%.3f/%.3f ms (%d/%d) %s %s  %d=%d:%d:%d:%d:%d:%d:%d:%d\n";
 
 const char report_sum_bw_read_enhanced_format[] =
 "[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d=%d:%d:%d:%d:%d:%d:%d:%d\n";
 
 const char report_triptime_enhanced_format[] =
-"[%3d] " IPERFTimeFrmt " trip-time (3WHS done->fin+finack) = %4.4f sec\n";
+"%s" IPERFTimeFrmt " trip-time (3WHS done->fin+finack) = %.4f sec\n";
 
 #ifdef HAVE_STRUCT_TCP_INFO_TCPI_TOTAL_RETRANS
 const char report_bw_write_enhanced_header[] =
 "[ ID] Interval" IPERFTimeSpace "Transfer    Bandwidth       Write/Err  Rtry     Cwnd/RTT        NetPwr\n";
 
-const char report_bw_write_enhanced_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %10d %8dK/%u us  %4.2f\n";
-
-const char report_bw_write_enhanced_nocwnd_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %10d       NA/%u us  %4.2f\n";
-
 const char report_sum_bw_write_enhanced_format[] =
 "[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d%10d\n";
 
-const char report_sumcnt_bw_write_enhanced_format[] =
-"[SUM-%d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d%10d\n";
+const char report_bw_write_enhanced_format[] =
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %10d %8dK/%u us  %s\n";
+
+const char report_bw_write_enhanced_nocwnd_format[] =
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %10d       NA/%u us  %s\n";
+
+const char report_write_enhanced_isoch_header[] =
+"[ ID] Interval" IPERFTimeSpace "Transfer     Bandwidth      Write/Err  Rtry     Cwnd/RTT     isoch:tx/miss/slip  NetPwr\n";
+
+const char report_write_enhanced_isoch_format[] =
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %8d %8dK/%u us  %9" PRIuMAX "/%" PRIuMAX "/%" PRIuMAX "        %s\n";
+
+const char report_write_enhanced_isoch_nocwnd_format[] =
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %10d    NA/%u us  %9" PRIuMAX "/%" PRIuMAX "/%" PRIuMAX " %s\n";
 
 #else
 const char report_bw_write_enhanced_header[] =
 "[ ID] Interval" IPERFTimeSpace "Transfer    Bandwidth       Write/Err\n";
 
 const char report_bw_write_enhanced_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d\n";
 
 const char report_sum_bw_write_enhanced_format[] =
 "[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d\n";
 
-const char report_sumcnt_bw_write_enhanced_format[] =
-"[SUM-%d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d\n";
+const char report_write_enhanced_isoch_header[] =
+"[ ID] Interval" IPERFTimeSpace "Transfer     Bandwidth      Write/Err  isoch:tx/miss/slip\n";
+
+const char report_write_enhanced_isoch_format[] =
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %9" PRIuMAX "/%" PRIuMAX "/%" PRIuMAX "\n";
 
 #endif
 
-const char report_bw_write_sumcnt_enhanced_header[] =
+const char report_sumcnt_bw_write_enhanced_header[] =
 "[SUM-cnt] Interval" IPERFTimeSpace "Transfer    Bandwidth       Write/Err  Rtry\n";
 
+const char report_sumcnt_bw_write_enhanced_format[] =
+"[SUM-%d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d%10d\n";
+
 const char report_bw_pps_enhanced_header[] =
 "[ ID] Interval" IPERFTimeSpace "Transfer     Bandwidth      Write/Err  PPS\n";
 
 const char report_bw_pps_enhanced_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %8.0f pps\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %8.0f pps\n";
+
+const char report_sumcnt_bw_pps_enhanced_header[] =
+"[SUM-cnt] Interval" IPERFTimeSpace "Transfer     Bandwidth      Write/Err  PPS\n";
+
+const char report_sumcnt_bw_pps_enhanced_format[] =
+"[SUM-%d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %8.0f pps\n";
 
 const char report_bw_pps_enhanced_isoch_header[] =
-"[ ID] Interval" IPERFTimeSpace "Transfer     Bandwidth      Write/Err  PPS  frames:tx/missed/slips\n";
+"[ ID] Interval" IPERFTimeSpace "Transfer     Bandwidth      Write/Err  PPS  isoch:tx/miss/slip\n";
 
 const char report_bw_pps_enhanced_isoch_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %8.0f pps  %3d/%d/%d\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %8.0f pps  %3" PRIuMAX "/%" PRIuMAX "/%" PRIuMAX "\n";
 
 const char report_sum_bw_pps_enhanced_format[] =
 "[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec  %d/%d %8.0f pps\n";
@@ -354,54 +405,74 @@ const char report_bw_jitter_loss_pps_header[] =
 "[ ID] Interval       Transfer     Bandwidth        Jitter   Lost/Total Datagrams   PPS\n";
 
 const char report_bw_jitter_loss_pps_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%) %8.0f pps\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %" PRIdMAX "/%" PRIdMAX " (%.2g%%) %8.0f pps\n";
 
 const char report_bw_jitter_loss_enhanced_header[] =
 "[ ID] Interval" IPERFTimeSpace "Transfer     Bandwidth        Jitter   Lost/Total \
- Latency avg/min/max/stdev PPS  inP NetPwr\n";
+ Latency avg/min/max/stdev PPS NetPwr\n";
 
 const char report_bw_jitter_loss_enhanced_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%) %6.3f/%6.3f/%6.3f/%6.3f ms %4.0f pps %s %4.2f\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %" PRIdMAX "/%" PRIdMAX " (%.2g%%) %.3f/%.3f/%.3f/%.3f ms %.0f pps %s\n";
+
+const char report_bw_jitter_loss_enhanced_triptime_header[] =
+"[ ID] Interval" IPERFTimeSpace "Transfer     Bandwidth        Jitter   Lost/Total \
+ Latency avg/min/max/stdev PPS  Rx/inP  NetPwr\n";
+
+const char report_bw_jitter_loss_enhanced_triptime_format[] =
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %" PRIdMAX "/%" PRIdMAX " (%.2g%%) %.3f/%.3f/%.3f/%.3f ms %.0f pps %s %s\n";
 
 const char report_bw_jitter_loss_enhanced_isoch_header[] =
 "[ ID] Interval" IPERFTimeSpace "Transfer     Bandwidth        Jitter   Lost/Total \
- Latency avg/min/max/stdev PPS  inP NetPwr  Frames/Lost\n";
+ Latency avg/min/max/stdev PPS  NetPwr  Isoch:rx/lost\n";
+
+const char report_bw_jitter_loss_enhanced_isoch_triptime_header[] =
+"[ ID] Interval" IPERFTimeSpace "Transfer     Bandwidth        Jitter   Lost/Total \
+ Latency avg/min/max/stdev PPS  Rx/inP NetPwr  Isoch:rx/lost\n";
 
 const char report_bw_jitter_loss_enhanced_isoch_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%) %6.3f/%6.3f/%6.3f/%6.3f ms %4.0f pps %s %4.2f  %3d/%d\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %" PRIdMAX "/%" PRIdMAX " (%.2g%%) %.3f/%.3f/%.3f/%.3f ms %.0f pps %s %s  %3d/%d\n";
 
 const char report_sum_bw_jitter_loss_enhanced_format[] =
-"[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%)  %4.0f pps\n";
+"[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %" PRIdMAX "/%" PRIdMAX " (%.2g%%)  %.0f pps\n";
 
 const char report_sumcnt_bw_jitter_loss_enhanced_format[] =
-"[SUM-%d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%)  %4.0f pps\n";
+"[SUM-%d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %" PRIdMAX "/%" PRIdMAX " (%.2g%%)  %.0f pps\n";
 
 const char report_bw_jitter_loss_suppress_enhanced_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%) -/-/-/- ms %4.0f pps\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms %" PRIdMAX "/%" PRIdMAX " (%.2g%%) -/-/-/- ms %.0f pps\n";
 
 /*
  * Frame interval reports
  */
 #define IPERFFTimeFrmt "%4.4f-%4.4f"
-#define IPERFFTimeSpace "       "
+#define IPERFFTimeSpace "  "
 const char report_frame_jitter_loss_enhanced_header[] =
 "[ ID] Interval(f-transit)" IPERFFTimeSpace "Transfer     Bandwidth    FrameID   Jitter   Lost/Total \
  Latency avg/min/max/stdev PPS  inP NetPwr\n";
 
 const char report_frame_jitter_loss_enhanced_format[] =
-"[%3d] " IPERFFTimeFrmt "(%0.4f) sec %ss  %ss/sec %4" PRIdMAX "   %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%) %6.3f/%6.3f/%6.3f/%6.3f ms %4.0f pps %2.0f pkts %4.2f\n";
+"%s" IPERFFTimeFrmt "(%0.4f) sec %ss  %ss/sec %" PRIdMAX "   %6.3f ms %" PRIdMAX "/%" PRIdMAX " (%.2g%%) %.3f/%.3f/%.3f/%.3f ms %.0f pps %2.0f pkts %s\n";
 
 const char report_frame_jitter_loss_suppress_enhanced_format[] =
-"[%3d] " IPERFTimeFrmt "(%0.4f) sec %ld %ss  %ss/sec %4" PRIdMAX "   %6.3f ms %4" PRIdMAX "/%5" PRIdMAX " (%.2g%%) -/-/-/- ms %4.0f pps\n";
+"%s" IPERFTimeFrmt "(%0.4f) sec %" PRIdMAX " %ss  %ss/sec %" PRIdMAX "   %6.3f ms %" PRIdMAX "/%" PRIdMAX " (%.2g%%) -/-/-/- ms %.0f pps\n";
 
 const char report_frame_tcp_enhanced_header[] =
 "[ ID] Interval(f-transit)" IPERFFTimeSpace "Transfer     Bandwidth    FrameID\n";
 
+const char report_burst_read_tcp_header[] =
+"[ ID] Burst (start-end)" IPERFFTimeSpace "Transfer     Bandwidth       XferTime  (DC%)     Reads=Dist          NetPwr\n";
+
+const char report_burst_read_tcp_format[] =
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec  %6.3f ms (%.2g%%)    %d=%d:%d:%d:%d:%d:%d:%d:%d  %s\n";
+
+const char report_burst_read_tcp_final_format[] =
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec               %d=%d:%d:%d:%d:%d:%d:%d:%d\n";
+
 /* -------------------------------------------------------------------
  * Fullduplex reports
  * ------------------------------------------------------------------- */
 const char report_bw_sum_fullduplex_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec\n";
 
 const char report_bw_sum_fullduplex_enhanced_format[] =
 "[FD%d] " IPERFTimeFrmt " sec  %ss  %ss/sec\n";
@@ -409,38 +480,50 @@ const char report_bw_sum_fullduplex_enhanced_format[] =
 const char report_udp_fullduplex_header[] =
 "[ ID] Interval       Transfer     Bandwidth    Datagrams   PPS\n";
 
+const char report_sumcnt_udp_fullduplex_header[] =
+"[SUM-cnt] Interval       Transfer     Bandwidth    Datagrams   PPS\n";
+
+const char report_sumcnt_udp_fullduplex_format[] =
+"[SUM-%d] " IPERFTimeFrmt " sec  %ss  %ss/sec %" PRIdMAX "%8.0f pps\n";
+
 const char report_udp_fullduplex_format[] =
-"[%3d] " IPERFTimeFrmt " sec  %ss  %ss/sec %5" PRIdMAX "%8.0f pps\n";
+"%s" IPERFTimeFrmt " sec  %ss  %ss/sec %" PRIdMAX "%8.0f pps\n";
 
 const char report_udp_fullduplex_enhanced_format[] =
-"[FD%d] " IPERFTimeFrmt " sec  %ss  %ss/sec %5" PRIdMAX "%8.0f pps\n";
+"[FD%d] " IPERFTimeFrmt " sec  %ss  %ss/sec %" PRIdMAX "%8.0f pps\n";
 
 const char report_udp_fullduplex_sum_format[] =
-"[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec %5" PRIdMAX "%8.0f pps\n";
+"[SUM] " IPERFTimeFrmt " sec  %ss  %ss/sec %" PRIdMAX "%8.0f pps\n";
 
 /* -------------------------------------------------------------------
  * Misc reports
  * ------------------------------------------------------------------- */
 const char report_outoforder[] =
-"[%3d] " IPERFTimeFrmt " sec  %d datagrams received out-of-order\n";
+"%s" IPERFTimeFrmt " sec  %d datagrams received out-of-order\n";
+
+const char report_sumcnt_outoforder[] =
+"[SUM-%d] " IPERFTimeFrmt " sec  %d datagrams received out-of-order\n";
 
 const char report_l2statistics[] =
-"[%3d] " IPERFTimeFrmt " sec   L2 processing detected errors, total(length/checksum/unknown) = %" PRIdMAX "(%" PRIdMAX "/%" PRIdMAX "/%" PRIdMAX ")\n";
+"%s" IPERFTimeFrmt " sec   L2 processing detected errors, total(length/checksum/unknown) = %" PRIdMAX "(%" PRIdMAX "/%" PRIdMAX "/%" PRIdMAX ")\n";
 
 const char report_sum_outoforder[] =
 "[SUM] " IPERFTimeFrmt " sec  %d datagrams received out-of-order\n";
 
 const char report_peer [] =
-"[%3d] local %s port %u connected with %s port %u%s\n";
+"%slocal %s port %u connected with %s port %u%s\n";
 
 const char report_peer_dev [] =
-"[%3d] local %s%%%s port %u connected with %s port %u%s\n";
+"%slocal %s%%%s port %u connected with %s port %u%s\n";
+
+const char report_peer_fail [] =
+"[drop] local %s port %u connection with %s port %u%s (permit key fail)\n";
 
 const char report_mss_unsupported[] =
-"[%3d] MSS and MTU size unknown (TCP_MAXSEG not supported)\n";
+"%sMSS and MTU size unknown (TCP_MAXSEG not supported)\n";
 
 const char report_mss[] =
-"[%3d] MSS size %d bytes\n";
+"%sMSS size %d bytes\n";
 
 const char report_datagrams[] =
 "[%3d] Sent %d datagrams\n";
@@ -458,9 +541,11 @@ const char reportCSV_peer[] =
 "%s,%u,%s,%u";
 
 const char report_l2length_error[] =
-"[%3d] " IPERFTimeFrmt " sec  %d datagrams received out-of-order\n";
-
+"%s" IPERFTimeFrmt " sec  %d datagrams received out-of-order\n";
 
+/*-------------------------------------------------------------------
+ * CSV outputs
+ *------------------------------------------------------------------*/
 const char reportCSV_bw_format[] =
 "%s,%s,%d,%.1f-%.1f,%" PRIdMAX ",%" PRIdMAX "\n";
 
@@ -546,6 +631,15 @@ const char warn_len_too_small_peer_exchange[] =
 const char warn_compat_and_peer_exchange[] =
 "WARNING: Options of '-C' '--compatibility' AND '-X' '--peerdetect' are mutually exclusive, --peerdetect ignored\n";
 
+const char warn_start_before_now[] =
+"[%3d] WARNING: --txstart-time (%" PRIdMAX ".%" PRIdMAX ") %s is before now %s\n";
+
+const char error_starttime_exceeds[] =
+"ERROR: --txstart-time (%" PRIdMAX ".%" PRIdMAX ") %s exceeds max scheduled delay of %d secs\n";
+
+const char error_delaytime_exceeds[] =
+"ERROR: --txdelay-time of %d seconds is more than the supported delay of %d seconds\n";
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif
