@@ -58,18 +58,22 @@
 #include "Locale.h"
 #include "active_hosts.h"
 #include "payloads.h"
+static int transferid_counter = 0;
 
-static inline void my_str_copy(char **dst, char *src) {
+static inline int my_str_copy(char **dst, char *src) {
+    int cnt = 0;
     if (src) {
-	*dst = (char *) calloc(1, (strlen(src) + 1));
+        *dst = (char *) calloc(strlen(src) + 1, sizeof(char));
 	if (*dst == NULL) {
 	    fprintf(stderr, "Out of Memory!!\n");
 	    exit(1);
 	}
+	cnt = strlen(src) + 1;
         strcpy((*dst), src);
     } else {
 	*dst = NULL;
     }
+    return cnt;
 }
 
 // These are the thread settings that are shared among report types
@@ -77,20 +81,23 @@ static inline void my_str_copy(char **dst, char *src) {
 // better encpasulate report handling.
 static void common_copy (struct ReportCommon **common, struct thread_Settings *inSettings) {
     // Do deep copies from settings
-    *common = (struct ReportCommon *) calloc(1,sizeof(struct ReportCommon));
-#ifdef HAVE_THREAD_DEBUG
-    thread_debug("Alloc common=%p", (void *)(*common));
-#endif
-    my_str_copy(&(*common)->Host, inSettings->mHost);
-    my_str_copy(&(*common)->Localhost, inSettings->mLocalhost);
-    my_str_copy(&(*common)->Ifrname, inSettings->mIfrname);
-    my_str_copy(&(*common)->Ifrnametx, inSettings->mIfrnametx);
-    my_str_copy(&(*common)->SSMMulticastStr, inSettings->mSSMMulticastStr);
-    my_str_copy(&(*common)->Congestion, inSettings->mCongestion);
+    *common = (struct ReportCommon *) calloc(1, sizeof(struct ReportCommon));
+    int bytecnt = 0;
+    bytecnt = my_str_copy(&(*common)->Host, inSettings->mHost);
+    bytecnt += my_str_copy(&(*common)->Localhost, inSettings->mLocalhost);
+    bytecnt += my_str_copy(&(*common)->Ifrname, inSettings->mIfrname);
+    bytecnt += my_str_copy(&(*common)->Ifrnametx, inSettings->mIfrnametx);
+    bytecnt += my_str_copy(&(*common)->SSMMulticastStr, inSettings->mSSMMulticastStr);
+    bytecnt += my_str_copy(&(*common)->Congestion, inSettings->mCongestion);
+    bytecnt += my_str_copy(&(*common)->transferIDStr, inSettings->mTransferIDStr);
+    bytecnt += my_str_copy(&(*common)->PermitKey, inSettings->mPermitKey);
     // copy some relevant settings
     (*common)->flags = inSettings->flags;
     (*common)->flags_extend = inSettings->flags_extend;
+    (*common)->flags_extend2 = inSettings->flags_extend2;
     (*common)->ThreadMode = inSettings->mThreadMode;
+    (*common)->ReportMode = inSettings->mReportMode;
+    (*common)->KeyCheck = inSettings->mKeyCheck;
     (*common)->Format = inSettings->mFormat;
     (*common)->TTL = inSettings->mTTL;
     // copy some traffic related settings
@@ -99,11 +106,14 @@ static void common_copy (struct ReportCommon **common, struct thread_Settings *i
     (*common)->TCPWin = inSettings->mTCPWin;
     (*common)->FQPacingRate = inSettings->mFQPacingRate;
     (*common)->Port = inSettings->mPort;
+    (*common)->PortLast = inSettings->mPortLast;
     (*common)->BindPort = inSettings->mBindPort;
     (*common)->ListenPort = inSettings->mListenPort;
-    (*common)->UDPRate = inSettings->mUDPRate;
-    (*common)->UDPRateUnits = inSettings->mUDPRateUnits;
+    (*common)->AppRate = inSettings->mAppRate;
+    (*common)->BurstSize = inSettings->mBurstSize;
+    (*common)->AppRateUnits = inSettings->mAppRateUnits;
     (*common)->socket = inSettings->mSock;
+    (*common)->transferID = inSettings->mTransferID;
     (*common)->threads = inSettings->mThreads;
     (*common)->winsize_requested = inSettings->mTCPWin;
 #if defined(HAVE_LINUX_FILTER_H) && defined(HAVE_AF_PACKET)
@@ -117,6 +127,12 @@ static void common_copy (struct ReportCommon **common, struct thread_Settings *i
     (*common)->RXbinsize =inSettings->mRXbinsize;
     (*common)->RXunits =inSettings->mRXunits;
     (*common)->pktIPG =inSettings->mBurstIPG;
+    (*common)->rtt_weight =inSettings->rtt_nearcongest_divider;
+    (*common)->ListenerTimeout =inSettings->mListenerTimeout;
+    (*common)->FPS = inSettings->mFPS;
+#ifdef HAVE_THREAD_DEBUG
+    thread_debug("Alloc common rpt/com/size/strsz %p/%p/%d/%d", (void *) common, (void *)(*common), sizeof(struct ReportCommon), bytecnt);
+#endif
 }
 
 static void free_common_copy (struct ReportCommon *common) {
@@ -137,65 +153,125 @@ static void free_common_copy (struct ReportCommon *common) {
 	free(common->SSMMulticastStr);
     if (common->Congestion)
 	free(common->Congestion);
+    if (common->transferIDStr)
+	free(common->transferIDStr);
+    if (common->PermitKey)
+	free(common->PermitKey);
     free(common);
 }
 
-void SetSumHandlers (struct thread_Settings *inSettings, struct SumReport* sumreport, int fullduplex) {
-    if (fullduplex) {
-	if (isUDP(inSettings)) {
-	    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_fullduplex_udp;
-	    sumreport->info.output_handler = ((isSumOnly(inSettings) || (inSettings->mReportMode == kReport_CSV)) ? NULL : \
-					      (isEnhanced(inSettings) ? udp_output_fullduplex_enhanced : udp_output_fullduplex));
-	} else {
-	    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_fullduplex_tcp;
-	    sumreport->info.output_handler = ((isSumOnly(inSettings) || (inSettings->mReportMode == kReport_CSV)) ? NULL : \
-					      (isEnhanced(inSettings) ? tcp_output_fullduplex_enhanced : tcp_output_fullduplex));
+// This will set the transfer id and id string
+// on the setting object. If the current id is zero
+// this will get the next one. Otherwise it will use
+// the value.
+void setTransferID (struct thread_Settings *inSettings, int role_reversal) {
+    if (!inSettings->mTransferIDStr) {
+	if (!inSettings->mTransferID) {
+	    Mutex_Lock(&transferid_mutex);
+	    inSettings->mTransferID = ++transferid_counter;
+	    Mutex_Unlock(&transferid_mutex);
 	}
+	int len = 0;
+	if (role_reversal)  {
+#ifdef HAVE_ROLE_REVERSAL_ID
+	    if (isPermitKey(inSettings) && (inSettings->mPermitKey[0] != '\0')) {
+		len = snprintf(NULL, 0, "[%s(*%d)] ", \
+			       inSettings->mPermitKey, inSettings->mTransferID);
+		inSettings->mTransferIDStr = (char *) calloc(len + 1, sizeof(char));
+		len = sprintf(inSettings->mTransferIDStr, "[%s(*%d)] ", \
+			       inSettings->mPermitKey, inSettings->mTransferID);
+	    } else if (inSettings->mTransferID < 10) {
+		len = snprintf(NULL, 0, "[ *%d] ", inSettings->mTransferID);
+		inSettings->mTransferIDStr = (char *) calloc(len + 1, sizeof(char));
+		len = sprintf(inSettings->mTransferIDStr, "[ *%d] ", inSettings->mTransferID);
+	    } else {
+		len = snprintf(NULL, 0, "[*%d] ", inSettings->mTransferID);
+		inSettings->mTransferIDStr = (char *) calloc(len + 1, sizeof(char));
+		len = sprintf(inSettings->mTransferIDStr, "[*%d] ", inSettings->mTransferID);
+	    }
+#endif
+	} else if (isPermitKey(inSettings) && (inSettings->mPermitKey[0] != '\0')) {
+	    len = snprintf(NULL, 0, "[%s(%d)] ", \
+			   inSettings->mPermitKey, inSettings->mTransferID);
+	    inSettings->mTransferIDStr = (char *) calloc(len + 1, sizeof(char));
+	    len = sprintf(inSettings->mTransferIDStr, "[%s(%d)] ", \
+			   inSettings->mPermitKey, inSettings->mTransferID);
+	} else  {
+	    len = snprintf(NULL, 0, "[%3d] ", inSettings->mTransferID);
+	    inSettings->mTransferIDStr = (char *) calloc(len+1, sizeof(char));
+	    len = sprintf(inSettings->mTransferIDStr, "[%3d] ", inSettings->mTransferID);
+	}
+    }
+}
+
+void SetFullDuplexHandlers (struct thread_Settings *inSettings, struct SumReport* sumreport) {
+    if (isUDP(inSettings)) {
+	sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_fullduplex_udp;
+	sumreport->info.output_handler = ((inSettings->mReportMode == kReport_CSV) ? NULL : \
+					  (isSumOnly(inSettings) ? NULL : \
+					   (isEnhanced(inSettings) ? udp_output_fullduplex_enhanced : udp_output_fullduplex)));
     } else {
-	switch (inSettings->mThreadMode) {
-	case kMode_Server :
-	    if (isUDP(inSettings)) {
-		sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_server_udp;
-		if (isFullDuplex(inSettings)) {
-		    sumreport->info.output_handler = udp_output_fullduplex_sum;
-		} else {
-		    sumreport->info.output_handler =  (isSumOnly(inSettings) ? udp_output_sumcnt_read : udp_output_sum_read);
-		}
+	sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_fullduplex_tcp;
+	sumreport->info.output_handler = ((inSettings->mReportMode == kReport_CSV) ? NULL : \
+					      (isSumOnly(inSettings) ? NULL : \
+					       (isEnhanced(inSettings) ? tcp_output_fullduplex_enhanced : tcp_output_fullduplex)));
+    }
+}
+
+void SetSumHandlers (struct thread_Settings *inSettings, struct SumReport* sumreport) {
+    switch (inSettings->mThreadMode) {
+    case kMode_Server :
+	if (isUDP(inSettings)) {
+	    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_server_udp;
+	    if (isSumOnly(inSettings)) {
+		sumreport->info.output_handler = udp_output_sumcnt_enhanced;
+	    } else if (isFullDuplex(inSettings)) {
+		sumreport->info.output_handler = udp_output_fullduplex_sum;
 	    } else {
-		if (isEnhanced(inSettings)) {
-		    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_server_tcp;
-		    sumreport->info.output_handler = (isSumOnly(inSettings) ? tcp_output_sumcnt_read_enhanced : tcp_output_sum_read_enhanced);
-		} else {
-		    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_server_tcp;
-		    sumreport->info.output_handler = (isSumOnly(inSettings) ? tcp_output_sumcnt_read : tcp_output_sum_read);
-		}
+		sumreport->info.output_handler =  (isEnhanced(inSettings) ? udp_output_sum_read_enhanced : udp_output_sum_read);
 	    }
-	    break;
-	case kMode_Client :
-	    if (isUDP(inSettings)) {
-		sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_client_udp;
-		if (isFullDuplex(inSettings)) {
-		    sumreport->info.output_handler = udp_output_fullduplex_sum;
-		} else {
-		    sumreport->info.output_handler =  (isSumOnly(inSettings) ? udp_output_sumcnt_write : \
-						       (isEnhanced(inSettings) ? udp_output_sum_write_enhanced : udp_output_sum_write));
-		}
+	} else {
+	    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_server_tcp;
+	    if (isSumOnly(inSettings)) {
+		sumreport->info.output_handler = (isEnhanced(inSettings) ? tcp_output_sumcnt_read_enhanced : tcp_output_sumcnt_read);
+	    } else if (isFullDuplex(inSettings)) {
+		sumreport->info.output_handler = tcp_output_sum_read;
 	    } else {
-		sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_client_tcp;
-		sumreport->info.output_handler = (isSumOnly(inSettings) ? tcp_output_sumcnt_write : tcp_output_sum_write);
+		sumreport->info.output_handler =  (isEnhanced(inSettings) ? tcp_output_sum_read_enhanced : tcp_output_sum_read);
 	    }
-	    break;
-	default:
-	    FAIL(1, "SetSumReport", inSettings);
 	}
+	break;
+    case kMode_Client :
+	if (isUDP(inSettings)) {
+	    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_client_udp;
+	    if (isSumOnly(inSettings)) {
+		sumreport->info.output_handler = ((isEnhanced(inSettings) && !isFullDuplex(inSettings)) ? \
+						  udp_output_sumcnt_write_enhanced : udp_output_sumcnt);
+	    } else if (isFullDuplex(inSettings)) {
+		sumreport->info.output_handler = udp_output_fullduplex_sum;
+	    } else {
+		sumreport->info.output_handler = (isEnhanced(inSettings) ? udp_output_sum_write_enhanced : udp_output_sum_write);
+	    }
+	} else {
+	    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_client_tcp;
+	    if (isSumOnly(inSettings)) {
+		sumreport->info.output_handler = (isEnhanced(inSettings) ? tcp_output_sumcnt_write_enhanced : tcp_output_sumcnt_write);
+	    } else if (isFullDuplex(inSettings)) {
+		sumreport->info.output_handler = tcp_output_fullduplex_sum;
+	    } else {
+		sumreport->info.output_handler = (isEnhanced(inSettings) ? tcp_output_sum_write_enhanced : tcp_output_sum_write);
+	    }
+	}
+	break;
+    default:
+	FAIL(1, "SetSumReport", inSettings);
     }
     // overide output handlers when csv reporting set
     if (inSettings->mReportMode == kReport_CSV)
 	sumreport->info.output_handler = NULL;
-
 }
 
-struct SumReport* InitSumReport(struct thread_Settings *inSettings, int inID, int fullduplex) {
+struct SumReport* InitSumReport(struct thread_Settings *inSettings, int inID, int fullduplex_report) {
     struct SumReport *sumreport = (struct SumReport *) calloc(1, sizeof(struct SumReport));
     if (sumreport == NULL) {
 	FAIL(1, "Out of Memory!!\n", inSettings);
@@ -206,7 +282,6 @@ struct SumReport* InitSumReport(struct thread_Settings *inSettings, int inID, in
     sumreport->threads = 0;
     common_copy(&sumreport->info.common, inSettings);
     sumreport->info.groupID = inID;
-    sumreport->info.transferID = inSettings->mSock;
     sumreport->info.threadcnt = 0;
     // Only initialize the interval time here
     // The startTime and nextTime for summing reports will be set by
@@ -214,9 +289,10 @@ struct SumReport* InitSumReport(struct thread_Settings *inSettings, int inID, in
     if ((inSettings->mInterval) && (inSettings->mIntervalMode == kInterval_Time)) {
 	sumreport->info.ts.intervalTime.tv_sec = (long) (inSettings->mInterval / rMillion);
 	sumreport->info.ts.intervalTime.tv_usec = (long) (inSettings->mInterval % rMillion);
+	sumreport->info.ts.significant_partial = ((double) inSettings->mInterval * PARTIALPERCENT / rMillion) ;
     }
-    SetSumHandlers(inSettings, sumreport, fullduplex);
-    if (fullduplex) {
+    if (fullduplex_report) {
+	SetFullDuplexHandlers(inSettings, sumreport);
 	if (!isServerReverse(inSettings)) {
 	    sumreport->fullduplex_barrier.count = 0;
 	    Condition_Initialize(&sumreport->fullduplex_barrier.await);
@@ -225,14 +301,12 @@ struct SumReport* InitSumReport(struct thread_Settings *inSettings, int inID, in
 	    if (sumreport->fullduplex_barrier.timeout < MINBARRIERTIMEOUT)
 		sumreport->fullduplex_barrier.timeout = MINBARRIERTIMEOUT;
 	} else {
-	    if (isTripTime(inSettings)) {
-		sumreport->info.ts.startTime = inSettings->triptime_start;
-	    } else {
-		sumreport->info.ts.startTime = inSettings->accept_time;
-	    }
+	    sumreport->info.ts.startTime = inSettings->accept_time;
 	    sumreport->info.ts.nextTime = sumreport->info.ts.startTime;
 	    TimeAdd(sumreport->info.ts.nextTime, sumreport->info.ts.intervalTime);
 	}
+    } else {
+	SetSumHandlers(inSettings, sumreport);
     }
 #ifdef HAVE_THREAD_DEBUG
     thread_debug("Init sum report %p id=%d", (void *)sumreport, inID);
@@ -253,6 +327,7 @@ struct ConnectionInfo * InitConnectOnlyReport (struct thread_Settings *thread) {
     creport->connect_times.vd = 0;
     creport->connect_times.m2 = 0;
     creport->connect_times.mean = 0;
+    creport->txholdbacktime = thread->txholdback_timer;
     return creport;
 }
 
@@ -313,7 +388,7 @@ void FreeReport (struct ReportHeader *reporthdr) {
 #ifdef HAVE_THREAD_DEBUG
     char rs[REPORTTXTMAX];
     reporttype_text(reporthdr, &rs[0]);
-    thread_debug("Jobq *FREE* report %p (%s)", reporthdr, &rs[0]);
+    thread_debug("Jobq *FREE* report hdr/rpt %p/%p (%s)", (void *) reporthdr, (void *) reporthdr->this_report, &rs[0]);
 #endif
     switch (reporthdr->type) {
     case DATA_REPORT:
@@ -329,7 +404,7 @@ void FreeReport (struct ReportHeader *reporthdr) {
 	Free_srReport((struct TransferInfo *)reporthdr->this_report);
 	break;
     default:
-	fprintf(stderr, "Invalid report type in free\n");
+	fprintf(stderr, "Invalid report type in free (%x)\n", reporthdr->type);
 	assert(0);
 	break;
     }
@@ -386,7 +461,7 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
     if (reporthdr == NULL) {
 	FAIL(1, "Out of Memory!!\n", inSettings);
     }
-    reporthdr->this_report = (void *) calloc(1, sizeof(struct ReporterData));
+    reporthdr->this_report = calloc(1, sizeof(struct ReporterData));
     if (reporthdr->this_report == NULL) {
 	FAIL(1, "Out of Memory!!\n", inSettings);
     }
@@ -405,14 +480,22 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
     }
     // Copy common settings into the transfer report section
     common_copy(&ireport->info.common, inSettings);
-#ifdef HAVE_THREAD_DEBUG
-    thread_debug("Job report %p uses multireport %p and fullduplex report is %p (socket=%d)", (void *)reporthdr->this_report, (void *)inSettings->mSumReport, (void *)inSettings->mFullDuplexReport, inSettings->mSock);
-#endif
+    ireport->burst_boundary = false;
+    ireport->info.final = false;
+    ireport->info.burstid_transition = false;
     // Create a new packet ring which is used to communicate
     // packet stats from the traffic thread to the reporter
     // thread.  The reporter thread does all packet accounting
     ireport->packetring = packetring_init((inSettings->numreportstructs ? inSettings->numreportstructs : (isSingleUDP(inSettings) ? 40 : NUM_REPORT_STRUCTS)), \
 					  &ReportCond, (isSingleUDP(inSettings) ? NULL : &inSettings->awake_me));
+#ifdef HAVE_THREAD_DEBUG
+    char rs[REPORTTXTMAX];
+    reporttype_text(reporthdr, &rs[0]);
+    thread_debug("Init %s report hdr/rpt/com=%p/%p/%p multireport/fullduplex=%p/%p pring(bytes)/cond=%p(%d)/%p (socket=%d)", &rs[0], \
+		 (void *) reporthdr, (void *) ireport, (void *) ireport->info.common, \
+		 (void *) inSettings->mSumReport, (void *) inSettings->mFullDuplexReport, \
+		 (void *) ireport->packetring, ireport->packetring->bytes, (void *) ireport->packetring->awake_producer, inSettings->mSock);
+#endif
     if (inSettings->numreportstructs)
 	fprintf (stdout, "[%3d] NUM_REPORT_STRUCTS override from %d to %d\n", inSettings->mSock, NUM_REPORT_STRUCTS, inSettings->numreportstructs);
     ireport->info.csv_peer[0] = '\0';
@@ -421,31 +504,51 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
     // 1) packet_handler: does packet accounting per the test and protocol
     // 2) transfer_protocol_handler: performs output, e.g. interval reports, per the test and protocol
 
+    if (inSettings->mIntervalMode == kInterval_Time) {
+	ireport->info.ts.intervalTime.tv_sec = (long) (inSettings->mInterval / rMillion);
+	ireport->info.ts.intervalTime.tv_usec = (long) (inSettings->mInterval % rMillion);
+	ireport->transfer_interval_handler = reporter_condprint_time_interval_report;
+	ireport->info.ts.significant_partial = (double) inSettings->mInterval * PARTIALPERCENT / rMillion ;
+    }
     switch (inSettings->mThreadMode) {
     case kMode_Server :
 	if (isUDP(inSettings)) {
 	    ireport->packet_handler = reporter_handle_packet_server_udp;
 	    ireport->transfer_protocol_handler = reporter_transfer_protocol_server_udp;
-            if (inSettings->mReportMode == kReport_CSV) {
-		ireport->info.output_handler = udp_output_basic_csv;
-	    } else if (isSumOnly(inSettings)) {
-		ireport->info.output_handler = NULL;
-	    } else if (isTripTime(inSettings)) {
-		if (isIsochronous(inSettings))
-		    ireport->info.output_handler = udp_output_read_enhanced_triptime_isoch;
-		else
-		    ireport->info.output_handler = udp_output_read_enhanced_triptime;
-	    } else if (isEnhanced(inSettings)) {
-		ireport->info.output_handler = udp_output_read_enhanced_triptime;
-	    } else if (!isFullDuplex(inSettings)) {
-		ireport->info.output_handler = udp_output_read;
+	    if ((inSettings->mIntervalMode == kInterval_Frames) && isIsochronous(inSettings)) {
+		ireport->transfer_interval_handler = reporter_condprint_frame_interval_report_server_udp;
+		ireport->transfer_protocol_handler = reporter_transfer_protocol_server_udp;
 	    } else {
-		ireport->info.output_handler =  NULL;
+		ireport->transfer_protocol_handler = reporter_transfer_protocol_server_udp;
+		if (inSettings->mReportMode == kReport_CSV) {
+		    ireport->info.output_handler = udp_output_basic_csv;
+		} else if (isSumOnly(inSettings)) {
+		    ireport->info.output_handler = NULL;
+		} else if (isTripTime(inSettings)) {
+		    if (isIsochronous(inSettings))
+			ireport->info.output_handler = udp_output_read_enhanced_triptime_isoch;
+		    else
+			ireport->info.output_handler = udp_output_read_enhanced_triptime;
+		} else if (isEnhanced(inSettings)) {
+		    ireport->info.output_handler = udp_output_read_enhanced;
+		} else if (isFullDuplex(inSettings)) {
+		    ireport->info.output_handler = udp_output_read;
+		} else {
+		    ireport->info.output_handler = udp_output_read;
+		}
 	    }
-	} else {
+	} else {  // TCP case
 	    ireport->packet_handler = reporter_handle_packet_server_tcp;
 	    ireport->transfer_protocol_handler = reporter_transfer_protocol_server_tcp;
-            if (inSettings->mReportMode == kReport_CSV) {
+	    if (isPeriodicBurst(inSettings)) {
+		ireport->transfer_interval_handler = reporter_condprint_burst_interval_report_server_tcp;
+		ireport->info.output_handler = tcp_output_burst_read;
+		ireport->burst_boundary = true;
+	    } else if ((inSettings->mIntervalMode == kInterval_Frames) && isIsochronous(inSettings)) {
+		ireport->transfer_interval_handler = reporter_condprint_frame_interval_report_server_tcp;
+		ireport->info.output_handler = tcp_output_frame_read_triptime;
+		ireport->burst_boundary = true;
+	    } else if (inSettings->mReportMode == kReport_CSV) {
 		ireport->info.output_handler = tcp_output_basic_csv;
 	    } else if (isSumOnly(inSettings)) {
 		ireport->info.output_handler = NULL;
@@ -456,7 +559,7 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 	    } else if (!isFullDuplex(inSettings)) {
 		ireport->info.output_handler = tcp_output_read;
 	    } else {
-		ireport->info.output_handler = NULL ;
+		ireport->info.output_handler = tcp_output_read;
 	    }
 	}
 	break;
@@ -473,7 +576,7 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 	    } else if (isEnhanced(inSettings)) {
 		ireport->info.output_handler = udp_output_write_enhanced;
 	    } else if (isFullDuplex(inSettings)) {
-		ireport->info.output_handler = NULL;
+		ireport->info.output_handler = udp_output_write;
 	    } else {
 		ireport->info.output_handler = udp_output_write;
 	    }
@@ -483,19 +586,17 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 		ireport->info.output_handler = tcp_output_basic_csv;
 	    } else if (isSumOnly(inSettings)) {
 		ireport->info.output_handler = NULL;
-	    } else if (isEnhanced(inSettings) || isIsochronous(inSettings)) {
+	    } else if (isIsochronous(inSettings)) {
+		ireport->info.output_handler = tcp_output_write_enhanced_isoch;
+	    } else if (isEnhanced(inSettings)) {
 		ireport->info.output_handler = tcp_output_write_enhanced;
 	    } else if (isFullDuplex(inSettings)) {
-		ireport->info.output_handler = NULL;
+		ireport->info.output_handler = tcp_output_write;
 	    } else {
 		ireport->info.output_handler = tcp_output_write;
 	    }
 	}
 	break;
-    case kMode_WriteAckClient :
-	ireport->packet_handler = reporter_handle_packet_null;
-	ireport->transfer_protocol_handler = reporter_transfer_protocol_null;
-	break;
     case kMode_Unknown :
     case kMode_Reporter :
     case kMode_ReporterClient :
@@ -504,46 +605,20 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 	FAIL(1, "InitIndividualReport\n", inSettings);
     }
 
-#ifdef HAVE_THREAD_DEBUG
-    thread_debug("Init data report %p using packetring=%p cond=%p", \
-		 (void *)ireport, (void *)(ireport->packetring), (void *)(ireport->packetring->awake_producer));
-#endif
-    ireport->info.transferID = inSettings->mSock;
-
-    switch (inSettings->mIntervalMode) {
-    case kInterval_Time :
-	ireport->info.ts.intervalTime.tv_sec = (long) (inSettings->mInterval / rMillion);
-	ireport->info.ts.intervalTime.tv_usec = (long) (inSettings->mInterval % rMillion);
-	ireport->transfer_interval_handler = reporter_condprint_time_interval_report;
-	break;
-    case kInterval_Frames :
-	if (isUDP(inSettings)) {
-	    ireport->transfer_interval_handler = reporter_condprint_frame_interval_report_udp;
-	} else {
-	    if (isTripTime(inSettings) || isIsochronous(inSettings))
-		ireport->transfer_interval_handler = reporter_condprint_frame_interval_report_tcp;
-	    else
-		ireport->transfer_interval_handler = NULL;
-	}
-	break;
-    default :
-	ireport->transfer_interval_handler = NULL;
-	break;
-    }
     if (inSettings->mThreadMode == kMode_Server) {
 	ireport->info.sock_callstats.read.binsize = inSettings->mBufLen / 8;
 	if (isRxHistogram(inSettings) && isUDP(inSettings) && isTripTime(inSettings)) {
 	    char name[] = "T8";
 	    ireport->info.latency_histogram =  histogram_init(inSettings->mRXbins,inSettings->mRXbinsize,0,\
 							      pow(10,inSettings->mRXunits), \
-							      inSettings->mRXci_lower, inSettings->mRXci_upper, ireport->info.transferID, name);
+							      inSettings->mRXci_lower, inSettings->mRXci_upper, ireport->info.common->transferID, name);
 	}
-	if (isRxHistogram(inSettings) && isIsochronous(inSettings) && isTripTime(inSettings)) {
+	if (isRxHistogram(inSettings) && (isIsochronous(inSettings) || (!isUDP(inSettings) && isTripTime(inSettings)))) {
 	    char name[] = "F8";
 	    // make sure frame bin size min is 100 microsecond
 	    ireport->info.framelatency_histogram =  histogram_init(inSettings->mRXbins,inSettings->mRXbinsize,0, \
 								   pow(10,inSettings->mRXunits), inSettings->mRXci_lower, \
-								   inSettings->mRXci_upper, ireport->info.transferID, name);
+								   inSettings->mRXci_upper, ireport->info.common->transferID, name);
 	}
     }
     return reporthdr;
@@ -571,11 +646,11 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
  */
 struct ReportHeader* InitConnectionReport (struct thread_Settings *inSettings, double ct) {
     assert(inSettings != NULL);
-    struct ReportHeader *reporthdr = calloc(sizeof(struct ReportHeader), sizeof(char*));
+    struct ReportHeader *reporthdr = (struct ReportHeader *) calloc(1, sizeof(struct ReportHeader));
     if (reporthdr == NULL) {
 	FAIL(1, "Out of Memory!!\n", inSettings);
     }
-    reporthdr->this_report = (void *) calloc(1, sizeof(struct ConnectionInfo));
+    reporthdr->this_report = calloc(1, sizeof(struct ConnectionInfo));
     if (reporthdr->this_report == NULL) {
 	FAIL(1, "Out of Memory!!\n", inSettings);
     }
@@ -584,10 +659,12 @@ struct ReportHeader* InitConnectionReport (struct thread_Settings *inSettings, d
 
     struct ConnectionInfo * creport = (struct ConnectionInfo *)(reporthdr->this_report);
     common_copy(&creport->common, inSettings);
-    if (!isUDP(inSettings) && (inSettings->mSock > 0))
+    if (!isUDP(inSettings) && (inSettings->mSock > 0) && !isDontRoute(inSettings) && \
+	!(ct <= 0.0 && (inSettings->mThreadMode == kMode_Client))) {
 	creport->MSS = getsock_tcp_mss(inSettings->mSock);
-    else
+    } else {
 	creport->MSS = -1;
+    }
     // Fill out known fields for the connection report
     reporter_peerversion(creport, inSettings->peer_version_u, inSettings->peer_version_l);
     creport->connecttime = ct;
@@ -609,8 +686,15 @@ struct ReportHeader* InitConnectionReport (struct thread_Settings *inSettings, d
                   (inSettings->mThreadMode != kMode_Client ? 0 : 1) );
     }
     creport->common->winsize_requested = inSettings->mTCPWin;
+    creport->txholdbacktime = inSettings->txholdback_timer;
+    if (isPeriodicBurst(inSettings)) {
+	creport->common->FPS = inSettings->mFPS;
+    }
 #ifdef HAVE_THREAD_DEBUG
-    thread_debug("Init connection report %p", reporthdr);
+    char rs[REPORTTXTMAX];
+    reporttype_text(reporthdr, &rs[0]);
+    thread_debug("Init %s report hdr/rpt/com %p/%p/%p", &rs[0],		\
+		 (void *) reporthdr, (void *) reporthdr->this_report, (void *) creport->common);
 #endif
     return reporthdr;
 }
@@ -621,11 +705,11 @@ struct ReportHeader* InitConnectionReport (struct thread_Settings *inSettings, d
  */
 struct ReportHeader *InitSettingsReport (struct thread_Settings *inSettings) {
     assert(inSettings != NULL);
-    struct ReportHeader *reporthdr = calloc(sizeof(struct ReportHeader), sizeof(char*));
+    struct ReportHeader *reporthdr = (struct ReportHeader *) calloc(1, sizeof(struct ReportHeader));
     if (reporthdr == NULL) {
 	FAIL(1, "Out of Memory!!\n", inSettings);
     }
-    reporthdr->this_report = (void *) calloc(1, sizeof(struct ReportSettings));
+    reporthdr->this_report = calloc(1, sizeof(struct ReportSettings));
     if (reporthdr->this_report == NULL) {
 	FAIL(1, "Out of Memory!!\n", inSettings);
     }
@@ -644,7 +728,10 @@ struct ReportHeader *InitSettingsReport (struct thread_Settings *inSettings) {
     sreport->isochstats.mBurstIPG = (unsigned int) (inSettings->mBurstIPG*1000.0);
     sreport->isochstats.mBurstInterval = (unsigned int) (1 / inSettings->mFPS * 1000000);
 #ifdef HAVE_THREAD_DEBUG
-    thread_debug("Init settings report %p", reporthdr);
+    char rs[REPORTTXTMAX];
+    reporttype_text(reporthdr, &rs[0]);
+    thread_debug("Init %s report hdr/rpt/com %p/%p/%p", &rs[0], \
+		 (void *) reporthdr, (void *) reporthdr->this_report, (void *) sreport->common);
 #endif
     return reporthdr;
 }
@@ -662,7 +749,7 @@ struct ReportHeader* InitServerRelayUDPReport(struct thread_Settings *inSettings
     if (reporthdr == NULL) {
 	FAIL(1, "Out of Memory!!\n", inSettings);
     }
-    reporthdr->this_report = (void *) calloc(1, sizeof(struct ServerRelay));
+    reporthdr->this_report = calloc(1, sizeof(struct ServerRelay));
     if (!reporthdr->this_report) {
 	FAIL(1, "Out of Memory!!\n", inSettings);
     }
@@ -673,8 +760,8 @@ struct ReportHeader* InitServerRelayUDPReport(struct thread_Settings *inSettings
     reporthdr->ReportMode = inSettings->mReportMode;
     struct ServerRelay *sr_report = (struct ServerRelay *)reporthdr->this_report;
     common_copy(&sr_report->info.common, inSettings);
-    struct TransferInfo *stats = (struct TransferInfo *)&sr_report->info;
-    stats->transferID = inSettings->mSock;
+    struct TransferInfo *stats = &sr_report->info;
+    stats->common->transferID = inSettings->mTransferID;
 
     stats->jitter = ntohl(server->base.jitter1);
     stats->jitter += ntohl(server->base.jitter2) / (double)rMillion;
@@ -700,26 +787,25 @@ struct ReportHeader* InitServerRelayUDPReport(struct thread_Settings *inSettings
 	stats->cntOutofOrder = ntohl(server->base.outorder_cnt);
 	stats->cntDatagrams = ntohl(server->base.datagrams);
     }
-    if (isEnhanced(stats->common)) {
-	if ((flags & SERVER_HEADER_EXTEND) != 0) {
-	    stats->transit.minTransit = ntohl(server->extend.minTransit1);
-	    stats->transit.minTransit += ntohl(server->extend.minTransit2) / (double)rMillion;
-	    stats->transit.maxTransit = ntohl(server->extend.maxTransit1);
-	    stats->transit.maxTransit += ntohl(server->extend.maxTransit2) / (double)rMillion;
-	    stats->transit.sumTransit = ntohl(server->extend.sumTransit1);
-	    stats->transit.sumTransit += ntohl(server->extend.sumTransit2) / (double)rMillion;
-	    stats->transit.meanTransit = ntohl(server->extend.meanTransit1);
-	    stats->transit.meanTransit += ntohl(server->extend.meanTransit2) / (double)rMillion;
-	    stats->transit.m2Transit = ntohl(server->extend.m2Transit1);
-	    stats->transit.m2Transit += ntohl(server->extend.m2Transit2) / (double)rMillion;
-	    stats->transit.vdTransit = ntohl(server->extend.vdTransit1);
-	    stats->transit.vdTransit += ntohl(server->extend.vdTransit2) / (double)rMillion;
-	    stats->transit.cntTransit = ntohl(server->extend.cntTransit);
-	    stats->cntIPG = ntohl(server->extend.cntIPG);
-	    stats->IPGsum = ntohl(server->extend.IPGsum);
-	} else {
-	    unsetEnhanced(stats->common);
-	}
+    if ((flags & SERVER_HEADER_EXTEND) != 0) {
+	setEnhanced(stats->common);
+	stats->transit.minTransit = ntohl(server->extend.minTransit1);
+	stats->transit.minTransit += ntohl(server->extend.minTransit2) / (double)rMillion;
+	stats->transit.maxTransit = ntohl(server->extend.maxTransit1);
+	stats->transit.maxTransit += ntohl(server->extend.maxTransit2) / (double)rMillion;
+	stats->transit.sumTransit = ntohl(server->extend.sumTransit1);
+	stats->transit.sumTransit += ntohl(server->extend.sumTransit2) / (double)rMillion;
+	stats->transit.meanTransit = ntohl(server->extend.meanTransit1);
+	stats->transit.meanTransit += ntohl(server->extend.meanTransit2) / (double)rMillion;
+	stats->transit.m2Transit = ntohl(server->extend.m2Transit1);
+	stats->transit.m2Transit += ntohl(server->extend.m2Transit2) / (double)rMillion;
+	stats->transit.vdTransit = ntohl(server->extend.vdTransit1);
+	stats->transit.vdTransit += ntohl(server->extend.vdTransit2) / (double)rMillion;
+	stats->transit.cntTransit = ntohl(server->extend.cntTransit);
+	stats->cntIPG = ntohl(server->extend.cntIPG);
+	stats->IPGsum = ntohl(server->extend.IPGsum);
+    } else {
+	unsetEnhanced(stats->common);
     }
     sr_report->peer = inSettings->local;
     sr_report->size_peer = inSettings->size_local;
@@ -748,6 +834,8 @@ void write_UDP_AckFIN (struct TransferInfo *stats) {
 
 	UDP_Hdr = (struct UDP_datagram*) ackPacket;
 	int flags = HEADER_VERSION1;
+	if (isEnhanced(stats->common) || isTripTime(stats->common))
+	    flags |= SERVER_HEADER_EXTEND;
 #ifdef HAVE_INT64_T
 	flags |=  HEADER_SEQNO64B;
 #endif
@@ -770,7 +858,7 @@ void write_UDP_AckFIN (struct TransferInfo *stats) {
 	}
 	hdr->base.jitter1      = htonl((long) stats->jitter);
 	hdr->base.jitter2      = htonl((long) ((stats->jitter - (long)stats->jitter) * rMillion));
-	flags |=  SERVER_HEADER_EXTEND;
+
 	hdr->extend.minTransit1  = htonl((long) stats->transit.totminTransit);
 	hdr->extend.minTransit2  = htonl((long) ((stats->transit.totminTransit - (long)stats->transit.totminTransit) * rMillion));
 	hdr->extend.maxTransit1  = htonl((long) stats->transit.totmaxTransit);
@@ -799,10 +887,11 @@ void write_UDP_AckFIN (struct TransferInfo *stats) {
 #ifdef HAVE_THREAD_DEBUG
 	    thread_debug("UDP server send done-ack w/server-stats to client (sock=%d)", stats->common->socket);
 #endif
-	    write(((stats->common->socketdrop > 0) ? stats->common->socketdrop : stats->common->socket), ackPacket, ackpacket_length);
+	    rc = write(((stats->common->socketdrop > 0) ? stats->common->socketdrop : stats->common->socket), ackPacket, ackpacket_length);
 #else
-	    write(stats->common->socket, ackPacket, ackpacket_length);
+	    rc = write(stats->common->socket, ackPacket, ackpacket_length);
 #endif
+	    WARN_errno(rc < 0, "write-ackfin");
 	    // wait here is for silence, no more packets from the client
 	    fd_set readSet;
 	    FD_ZERO(&readSet);
