@@ -63,9 +63,9 @@
 #include "Locale.h"
 #include "delay.h"
 #include "PerfSocket.hpp"
-#include "Write_ack.hpp"
 #include "SocketAddr.h"
 #include "payloads.h"
+#include <cmath>
 #if defined(HAVE_LINUX_FILTER_H) && defined(HAVE_AF_PACKET)
 #include "checksums.h"
 #endif
@@ -77,7 +77,7 @@
 
 Server::Server (thread_Settings *inSettings) {
 #ifdef HAVE_THREAD_DEBUG
-    thread_debug("Server constructor with thread=%p multihdr=%p(sock=%d)", (void *) inSettings, (void *)inSettings->mSumReport, inSettings->mSock);
+    thread_debug("Server constructor with thread=%p sum=%p (sock=%d)", (void *) inSettings, (void *)inSettings->mSumReport, inSettings->mSock);
 #endif
     mSettings = inSettings;
     mBuf = NULL;
@@ -95,10 +95,10 @@ Server::Server (thread_Settings *inSettings) {
 	}
     }
 #endif
-    // initialize buffer, length checking done by the Listener
-    mBuf = new char[(mSettings->mBufLen > MINMBUFALLOCSIZE) ? mSettings->mBufLen : MINMBUFALLOCSIZE]; // defined in payloads.h
+    mBufLen = (mSettings->mBufLen > MINMBUFALLOCSIZE) ? mSettings->mBufLen : MINMBUFALLOCSIZE;
+    mBuf = new char[mBufLen];
     FAIL_errno(mBuf == NULL, "No memory for buffer\n", mSettings);
-    if (mSettings->mBufLen < (int) sizeof(UDP_datagram)) {
+    if (mSettings->mBufLen < static_cast<int>(sizeof(UDP_datagram))) {
 	fprintf(stderr, warn_buffer_too_small, mSettings->mBufLen);
     }
 
@@ -110,29 +110,23 @@ Server::Server (thread_Settings *inSettings) {
     // mAmount integer, units 10 milliseconds
     // divide by two so timeout is 1/2 the interval
     if (mSettings->mInterval && (mSettings->mIntervalMode == kInterval_Time)) {
-	sorcvtimer = (mSettings->mInterval / 2);
+	sorcvtimer = static_cast<int>(round(1000000.0 * mSettings->mInterval / 2.0));
     } else if (isServerModeTime(mSettings)) {
-	sorcvtimer = (mSettings->mAmount * 1000) / 2;
+	sorcvtimer = static_cast<int>(round(mSettings->mAmount * 10000) / 2);
+    }
+    isburst = (isIsochronous(mSettings) || isPeriodicBurst(mSettings) || (isTripTime(mSettings) && !isUDP(mSettings)));
+    if (isburst && (mSettings->mFPS > 0.0)) {
+	sorcvtimer = static_cast<int>(round(2000000.0 / mSettings->mFPS));
     }
     if (sorcvtimer > 0) {
-#ifdef WIN32
-	// Windows SO_RCVTIMEO uses ms
-	DWORD timeout = (double) sorcvtimer / 1e3;
-#else
-	struct timeval timeout;
-	timeout.tv_sec = sorcvtimer / 1000000;
-	timeout.tv_usec = sorcvtimer % 1000000;
-#endif
-	if (setsockopt(mSettings->mSock, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(timeout)) < 0) {
-	    WARN_errno(mSettings->mSock == SO_RCVTIMEO, "socket");
-	}
+	SetSocketOptionsReceiveTimeout(mSettings, sorcvtimer);
     }
 }
 
 /* -------------------------------------------------------------------
  * Destructor close socket.
  * ------------------------------------------------------------------- */
-Server::~Server (void) {
+Server::~Server () {
 #if HAVE_THREAD_DEBUG
     thread_debug("Server destructor sock=%d fullduplex=%s", mySocket, (isFullDuplex(mSettings) ? "true" : "false"));
 #endif
@@ -146,11 +140,9 @@ Server::~Server (void) {
     DELETE_ARRAY(mBuf);
 }
 
-inline bool Server::InProgress (void) {
-    if (sInterupted || peerclose ||
-	((isServerModeTime(mSettings) || (isModeTime(mSettings) && isReverse(mSettings))) && mEndTime.before(reportstruct->packetTime)))
-	return false;
-    return true;
+inline bool Server::InProgress () {
+    return !(sInterupted || peerclose ||
+	((isServerModeTime(mSettings) || (isModeTime(mSettings) && isReverse(mSettings))) && mEndTime.before(reportstruct->packetTime)));
 }
 
 /* -------------------------------------------------------------------
@@ -158,17 +150,16 @@ inline bool Server::InProgress (void) {
  * Sends termination flag several times at the end.
  * Does not close the socket.
  * ------------------------------------------------------------------- */
-void Server::RunTCP (void) {
+void Server::RunTCP () {
     long currLen;
     intmax_t totLen = 0;
-    bool peerclose  = false;
     struct TCP_burst_payload burst_info;
     Timestamp time1, time2;
     double tokens=0.000004;
 
     if (!InitTrafficLoop())
 	return;
-    struct timeval prevsend = myReport->info.ts.startTime;
+    myReport->info.ts.prevsendTime = myReport->info.ts.startTime;
 
     int burst_nleft = 0;
     burst_info.burst_id = 0;
@@ -179,13 +170,14 @@ void Server::RunTCP (void) {
     now.setnow();
     reportstruct->packetTime.tv_sec = now.getSecs();
     reportstruct->packetTime.tv_usec = now.getUsecs();
-    while (InProgress() && !peerclose) {
+    while (InProgress()) {
+//	printf("***** bid expect = %u\n", burstid_expect);
 	reportstruct->emptyreport=1;
 	currLen = 0;
 	// perform read
 	if (isBWSet(mSettings)) {
 	    time2.setnow();
-	    tokens += time2.subSec(time1) * (mSettings->mUDPRate / 8.0);
+	    tokens += time2.subSec(time1) * (mSettings->mAppRate / 8.0);
 	    time1 = time2;
 	}
 	reportstruct->transit_ready = 0;
@@ -195,8 +187,8 @@ void Server::RunTCP (void) {
 	    if (burst_nleft > 0)
 		readLen = (mSettings->mBufLen < burst_nleft) ? mSettings->mBufLen : burst_nleft;
 	    reportstruct->emptyreport=1;
-	    if ((isIsochronous(mSettings) || isTripTime(mSettings)) && (burst_nleft == 0)) {
-		if ((n = recvn(mSettings->mSock, (char *)&burst_info, sizeof(struct TCP_burst_payload), 0)) == sizeof(struct TCP_burst_payload)) {
+	    if (isburst && (burst_nleft == 0)) {
+		if ((n = recvn(mSettings->mSock, reinterpret_cast<char *>(&burst_info), sizeof(struct TCP_burst_payload), 0)) == sizeof(struct TCP_burst_payload)) {
 		    // burst_info.typelen.type = ntohl(burst_info.typelen.type);
 		    // burst_info.typelen.length = ntohl(burst_info.typelen.length);
 		    burst_info.flags = ntohl(burst_info.flags);
@@ -204,7 +196,6 @@ void Server::RunTCP (void) {
 		    assert(burst_info.burst_size > 0);
 		    reportstruct->burstsize = burst_info.burst_size;
 		    burst_info.burst_id = ntohl(burst_info.burst_id);
-//		    printf("**** burst size = %d id = %d\n", burst_info.burst_size, burst_info.burst_id);
 		    reportstruct->frameID = burst_info.burst_id;
 		    if (isTripTime(mSettings)) {
 			reportstruct->sentTime.tv_sec = ntohl(burst_info.send_tt.write_tv_sec);
@@ -214,46 +205,48 @@ void Server::RunTCP (void) {
 			reportstruct->sentTime.tv_sec = now.getSecs();
 			reportstruct->sentTime.tv_usec = now.getUsecs();
 		    }
-		    prevsend = reportstruct->sentTime;
+		    // This is the first stamp of the burst
+		    myReport->info.ts.prevsendTime = reportstruct->sentTime;
 		    burst_nleft = burst_info.burst_size - n;
+		    if (burst_nleft == 0) {
+			reportstruct->prevSentTime = myReport->info.ts.prevsendTime;
+			reportstruct->transit_ready = 1;
+		    }
 		    currLen += n;
 		    readLen = (mSettings->mBufLen < burst_nleft) ? mSettings->mBufLen : burst_nleft;
 		    WARN(burst_nleft <= 0, "invalid burst read req size");
 		    // thread_debug("***read burst header size %d id=%d", burst_info.burst_size, burst_info.burst_id);
 		} else {
+		    if (n > 0)
+			WARN(1, "partial readn");
 #ifdef HAVE_THREAD_DEBUG
 		    thread_debug("TCP burst partial read of %d wanted %d", n, sizeof(struct TCP_burst_payload));
 #endif
-		    goto end;
+		    goto Done;
 		}
 	    }
-	    n = recv(mSettings->mSock, mBuf, readLen, 0);
-	    if (n > 0) {
-		reportstruct->emptyreport=0;
-		if (isIsochronous(mSettings) || isTripTime(mSettings)) {
-		    burst_nleft -= n;
-		    if (burst_nleft == 0) {
-#ifdef WRITEACKDONE
-		        if (isWriteAck(mSettings)) {
-			    enqueue_ackring(mSettings->ackring, reportstruct);
+	    if (!reportstruct->transit_ready) {
+		n = recv(mSettings->mSock, mBuf, readLen, 0);
+		if (n > 0) {
+		    reportstruct->emptyreport = 0;
+		    if (isburst) {
+			burst_nleft -= n;
+			if (burst_nleft == 0) {
+			    reportstruct->prevSentTime = myReport->info.ts.prevsendTime;
+			    reportstruct->transit_ready = 1;
 			}
-#endif
-			reportstruct->prevSentTime = prevsend;
-			reportstruct->transit_ready = 1;
-		    } else {
-//			printf("****currlen = %ld, n=%d, burst_nleft=%d id=%d\n", currLen, n, burst_nleft, burst_info.burst_id);
 		    }
-		}
-	    } else if (n == 0) {
-		peerclose = true;
+		} else if (n == 0) {
+		    peerclose = true;
 #ifdef HAVE_THREAD_DEBUG
-		thread_debug("Server thread detected EOF on socket %d", mSettings->mSock);
+		    thread_debug("Server thread detected EOF on socket %d", mSettings->mSock);
 #endif
-	    } else if ((n < 0) && (!NONFATALTCPREADERR(errno))) {
-		WARN_errno(1, "recv");
-		n = 0;
+		} else if ((n < 0) && (!NONFATALTCPREADERR(errno))) {
+		    // WARN_errno(1, "recv");
+		    n = 0;
+		}
+		currLen += n;
 	    }
-	    currLen += n;
 	    now.setnow();
 	    reportstruct->packetTime.tv_sec = now.getSecs();
 	    reportstruct->packetTime.tv_usec = now.getUsecs();
@@ -262,12 +255,15 @@ void Server::RunTCP (void) {
 		tokens -= currLen;
 
 	    reportstruct->packetLen = currLen;
+#ifdef HAVE_STRUCT_TCP_INFO_TCPI_TOTAL_RETRANS
+	    ReportPacket(myReport, reportstruct, NULL);
+#else
 	    ReportPacket(myReport, reportstruct);
-
+#endif
 	    // Check for reverse and amount where
 	    // the server stops after receiving
 	    // the expected byte count
-	    if (isReverse(mSettings) && !isModeTime(mSettings) && (totLen >= (intmax_t) mSettings->mAmount)) {
+	    if (isReverse(mSettings) && !isModeTime(mSettings) && (totLen >= static_cast<intmax_t>(mSettings->mAmount))) {
 	        break;
 	    }
 	} else {
@@ -275,7 +271,7 @@ void Server::RunTCP (void) {
 	    delay_loop(4);
 	}
     }
-  end:
+  Done:
     disarm_itimer();
     // stop timing
     now.setnow();
@@ -289,11 +285,11 @@ void Server::RunTCP (void) {
 	int rc = close(mySocket);
 	WARN_errno(rc == SOCKET_ERROR, "server close");
     }
-    Iperf_remove_host(&mSettings->peer);
+    Iperf_remove_host(mSettings);
     FreeReport(myJob);
 }
 
-void Server::InitKernelTimeStamping (void) {
+void Server::InitKernelTimeStamping () {
 #if HAVE_DECL_SO_TIMESTAMP
     iov[0].iov_base=mBuf;
     iov[0].iov_len=mSettings->mBufLen;
@@ -307,7 +303,7 @@ void Server::InitKernelTimeStamping (void) {
     message.msg_controllen = sizeof(ctrl);
 
     int timestampOn = 1;
-    if (setsockopt(mSettings->mSock, SOL_SOCKET, SO_TIMESTAMP, (int *) &timestampOn, sizeof(timestampOn)) < 0) {
+    if (setsockopt(mSettings->mSock, SOL_SOCKET, SO_TIMESTAMP, &timestampOn, sizeof(timestampOn)) < 0) {
 	WARN_errno(mSettings->mSock == SO_TIMESTAMP, "socket");
     }
 #endif
@@ -317,7 +313,7 @@ void Server::InitKernelTimeStamping (void) {
 // Set the report start times and next report times, options
 // are now, the accept time or the first write time
 //
-inline void Server::SetFullDuplexReportStartTime (void) {
+inline void Server::SetFullDuplexReportStartTime () {
     assert(myReport->FullDuplexReport != NULL);
     struct TransferInfo *fullduplexstats = &myReport->FullDuplexReport->info;
     assert(fullduplexstats != NULL);
@@ -331,22 +327,17 @@ inline void Server::SetFullDuplexReportStartTime (void) {
     thread_debug("Server fullduplex report start=%ld.%ld next=%ld.%ld", fullduplexstats->ts.startTime.tv_sec, fullduplexstats->ts.startTime.tv_usec, fullduplexstats->ts.nextTime.tv_sec, fullduplexstats->ts.nextTime.tv_usec);
 #endif
 }
-inline void Server::SetReportStartTime (void) {
-    if (isTripTime(mSettings)) {
-	// Start times come from the sender's timestamp
-	assert(mSettings->triptime_start.tv_sec != 0);
-	assert(mSettings->triptime_start.tv_usec != 0);
-	myReport->info.ts.startTime.tv_sec = mSettings->triptime_start.tv_sec;
-	myReport->info.ts.startTime.tv_usec = mSettings->triptime_start.tv_usec;
-	myReport->info.ts.prevpacketTime = myReport->info.ts.startTime;
-    } else if (TimeZero(myReport->info.ts.startTime) && !TimeZero(mSettings->accept_time)) {
-	// Servers that aren't full duplex use the accept timestamp for start
-	myReport->info.ts.startTime.tv_sec = mSettings->accept_time.tv_sec;
-	myReport->info.ts.startTime.tv_usec = mSettings->accept_time.tv_usec;
-    } else {
-	now.setnow();
-	myReport->info.ts.startTime.tv_sec = now.getSecs();
-	myReport->info.ts.startTime.tv_usec = now.getUsecs();
+inline void Server::SetReportStartTime () {
+    if (TimeZero(myReport->info.ts.startTime)) {
+	if (!TimeZero(mSettings->accept_time) && !isTxStartTime(mSettings)) {
+	    // Servers that aren't full duplex use the accept timestamp for start
+	    myReport->info.ts.startTime.tv_sec = mSettings->accept_time.tv_sec;
+	    myReport->info.ts.startTime.tv_usec = mSettings->accept_time.tv_usec;
+	} else {
+	    now.setnow();
+	    myReport->info.ts.startTime.tv_sec = now.getSecs();
+	    myReport->info.ts.startTime.tv_usec = now.getUsecs();
+	}
     }
     myReport->info.ts.IPGstart = myReport->info.ts.startTime;
 
@@ -371,9 +362,9 @@ inline void Server::SetReportStartTime (void) {
 #endif
 }
 
-bool Server::InitTrafficLoop (void) {
+bool Server::InitTrafficLoop () {
     myJob = InitIndividualReport(mSettings);
-    myReport = (struct ReporterData *)myJob->this_report;
+    myReport = static_cast<struct ReporterData *>(myJob->this_report);
     assert(myJob != NULL);
     //  copy the thread drop socket to this object such
     //  that the destructor can close it if needed
@@ -393,55 +384,63 @@ bool Server::InitTrafficLoop (void) {
 	if ((setfullduplexflag = fullduplex_start_barrier(&mSettings->mFullDuplexReport->fullduplex_barrier)) < 0)
 	    exit(-1);
     }
+
+    // Handle the case when the client spawns a server (no listener) and need the initial header
     // Case of --trip-times and --reverse or --fullduplex, listener handles normal case
-    if (isTripTime(mSettings) && TimeZero(mSettings->triptime_start)) {
+    if (isReverse(mSettings) && (isTripTime(mSettings) || isPeriodicBurst(mSettings))) {
 	int n = 0;
 	uint32_t flags = 0;
 	int peeklen = 0;
 	if (isUDP(mSettings)) {
-	    n = recvn(mSettings->mSock, mBuf, sizeof(uint32_t) + sizeof(struct UDP_datagram), MSG_PEEK);
+	    n = recvn(mSettings->mSock, mBuf, mBufLen, MSG_PEEK);
 	    if (n == 0) {
 		//peer closed the socket, with no writes e.g. a connect-only test
-		return -1;
+		return false;
 	    }
-	    FAIL_errno(n != (sizeof(uint32_t) + sizeof(struct UDP_datagram)), "read udp flags", mSettings);
-	    struct client_udp_testhdr *udp_pkt = (struct client_udp_testhdr *) mBuf;
+	    FAIL_errno(n < 0, "recvn-reverse", mSettings);
+	    struct client_udp_testhdr *udp_pkt = reinterpret_cast<struct client_udp_testhdr *>(mBuf);
 	    flags = ntohl(udp_pkt->base.flags);
-	    if ((peeklen = Settings_ClientHdrPeekLen(flags) + sizeof(struct UDP_datagram)) > 0) {
-		n = recvn(mSettings->mSock, mBuf, peeklen, MSG_PEEK);
-		FAIL_errno((n < peeklen), "read udp test hdr", mSettings);
-	    }
-	    mSettings->triptime_start.tv_sec = ntohl(udp_pkt->start_fq.start_tv_sec);
-	    mSettings->triptime_start.tv_usec = ntohl(udp_pkt->start_fq.start_tv_usec);
+	    mSettings->accept_time.tv_sec = ntohl(udp_pkt->start_fq.start_tv_sec);
+	    mSettings->accept_time.tv_usec = ntohl(udp_pkt->start_fq.start_tv_usec);
+	    reportstruct->packetLen = n;
 	} else {
 	    n = recvn(mSettings->mSock, mBuf, sizeof(uint32_t), MSG_PEEK);
 	    if (n == 0) {
 		fprintf(stderr, "WARN: zero read on header flags\n");
 		//peer closed the socket, with no writes e.g. a connect-only test
-		return -1;
+		return false;
 	    }
 	    FAIL_errno((n < (int) sizeof(uint32_t)), "read tcp flags", mSettings);
-	    struct client_tcp_testhdr *tcp_pkt = (struct client_tcp_testhdr *) mBuf;
+	    struct client_tcp_testhdr *tcp_pkt = reinterpret_cast<struct client_tcp_testhdr *>(mBuf);
 	    flags = ntohl(tcp_pkt->base.flags);
 	    // figure out the length of the test header
 	    if ((peeklen = Settings_ClientHdrPeekLen(flags)) > 0) {
 		// read the test settings passed to the mSettings by the client
 		n = recvn(mSettings->mSock, mBuf, peeklen, 0);
 		FAIL_errno((n < peeklen), "read tcp test info", mSettings);
-		struct client_tcp_testhdr *tcp_pkt = (struct client_tcp_testhdr *) mBuf;
-		mSettings->triptime_start.tv_sec = ntohl(tcp_pkt->start_fq.start_tv_sec);
-		mSettings->triptime_start.tv_usec = ntohl(tcp_pkt->start_fq.start_tv_usec);
-		reportstruct->packetID = n;
+		struct client_tcp_testhdr *tcp_pkt = reinterpret_cast<struct client_tcp_testhdr *>(mBuf);
+		mSettings->accept_time.tv_sec = ntohl(tcp_pkt->start_fq.start_tv_sec);
+		mSettings->accept_time.tv_usec = ntohl(tcp_pkt->start_fq.start_tv_usec);
+		reportstruct->packetLen = n;
+		mSettings->skip	= n;
+		if (n == 0)
+		    return false;
+		reportstruct->packetID = (n > 0) ? 1 : 0;
 	    }
 	}
     }
     if (isTripTime(mSettings)) {
 	Timestamp now;
-	if ((abs(now.getSecs() - mSettings->triptime_start.tv_sec)) > MAXDIFFTIMESTAMPSECS) {
+	if ((abs(now.getSecs() - mSettings->accept_time.tv_sec)) > MAXDIFFTIMESTAMPSECS) {
 	    unsetTripTime(mSettings);
 	    fprintf(stdout,"WARN: ignore --trip-times because client didn't provide valid start timestamp within %d seconds of now\n", MAXDIFFTIMESTAMPSECS);
 	}
     }
+    // skip the test exchange header to get to the first burst
+    // The test exchange header was read in listener context
+    if (mSettings->skip && (isTripTime(mSettings) || isPeriodicBurst(mSettings))) {
+	reportstruct->packetLen = recvn(mSettings->mSock, mBuf, mSettings->skip, 0);
+    }
     SetReportStartTime();
     if (setfullduplexflag)
 	SetFullDuplexReportStartTime();
@@ -453,8 +452,8 @@ bool Server::InitTrafficLoop (void) {
         int err;
         struct itimerval it;
 	memset (&it, 0, sizeof (it));
-	it.it_value.tv_sec = (int) (mSettings->mAmount / 100.0);
-	it.it_value.tv_usec = (int) (10000 * (mSettings->mAmount -
+	it.it_value.tv_sec = static_cast<int>(mSettings->mAmount / 100.0);
+	it.it_value.tv_usec = static_cast<int>(10000 * (mSettings->mAmount -
 					      it.it_value.tv_sec * 100.0));
 	err = setitimer(ITIMER_REAL, &it, NULL);
 	FAIL_errno(err != 0, "setitimer", mSettings);
@@ -472,17 +471,21 @@ bool Server::InitTrafficLoop (void) {
 	reportstruct->sentTime.tv_sec = myReport->info.ts.startTime.tv_sec;
 	reportstruct->sentTime.tv_usec = myReport->info.ts.startTime.tv_usec;
 	reportstruct->packetTime = reportstruct->sentTime;
+#ifdef HAVE_STRUCT_TCP_INFO_TCPI_TOTAL_RETRANS
+	ReportPacket(myReport, reportstruct, NULL);
+#else
 	ReportPacket(myReport, reportstruct);
+#endif
     }
     return true;
 }
 
-inline int Server::ReadWithRxTimestamp (void) {
+inline int Server::ReadWithRxTimestamp () {
     long currLen;
     int tsdone = 0;
 
 #if HAVE_DECL_SO_TIMESTAMP
-    cmsg = (struct cmsghdr *) &ctrl;
+    cmsg = reinterpret_cast<struct cmsghdr *>(&ctrl);
     currLen = recvmsg(mSettings->mSock, &message, mSettings->recvflags);
     if (currLen > 0) {
 	if (cmsg->cmsg_level == SOL_SOCKET &&
@@ -502,16 +505,17 @@ inline int Server::ReadWithRxTimestamp (void) {
 	    peerclose = true;
 	} else if
 #ifdef WIN32
-	    (WSAGetLastError() != WSAEWOULDBLOCK)
+		(WSAGetLastError() != WSAEWOULDBLOCK)
 #else
 	    ((errno != EAGAIN) && (errno != EWOULDBLOCK) && (errno != ECONNREFUSED))
 #endif
-        {
-	    WARN_errno(currLen, "recvmsg");
-	    currLen= 0;
-	}
+    {
+	WARN_errno(currLen, "recvmsg");
+	currLen= 0;
+    }
+    } else if (TimeZero(myReport->info.ts.prevpacketTime)) {
+	myReport->info.ts.prevpacketTime = reportstruct->packetTime;
     }
-
     if (!tsdone) {
 	now.setnow();
 	reportstruct->packetTime.tv_sec = now.getSecs();
@@ -521,16 +525,16 @@ inline int Server::ReadWithRxTimestamp (void) {
 }
 
 // Returns true if the client has indicated this is the final packet
-inline bool Server::ReadPacketID (void) {
+inline bool Server::ReadPacketID () {
     bool terminate = false;
-    struct UDP_datagram* mBuf_UDP  = (struct UDP_datagram*) (mBuf + mSettings->l4payloadoffset);
+    struct UDP_datagram* mBuf_UDP  = reinterpret_cast<struct UDP_datagram*>(mBuf + mSettings->l4payloadoffset);
 
     // terminate when datagram begins with negative index
     // the datagram ID should be correct, just negated
 
     if (isSeqNo64b(mSettings)) {
       // New client - Signed PacketID packed into unsigned id2,id
-      reportstruct->packetID = ((uint32_t)ntohl(mBuf_UDP->id)) | ((uintmax_t)(ntohl(mBuf_UDP->id2)) << 32);
+      reportstruct->packetID = (static_cast<uint32_t>(ntohl(mBuf_UDP->id))) | (static_cast<uintmax_t>(ntohl(mBuf_UDP->id2)) << 32);
 
 #ifdef HAVE_PACKET_DEBUG
       printf("id 0x%x, 0x%x -> %" PRIdMAX " (0x%" PRIxMAX ")\n",
@@ -538,7 +542,7 @@ inline bool Server::ReadPacketID (void) {
 #endif
     } else {
       // Old client - Signed PacketID in Signed id
-      reportstruct->packetID = (int32_t)ntohl(mBuf_UDP->id);
+      reportstruct->packetID = static_cast<int32_t>(ntohl(mBuf_UDP->id));
 #ifdef HAVE_PACKET_DEBUG
       printf("id 0x%x -> %" PRIdMAX " (0x%" PRIxMAX ")\n",
 	     ntohl(mBuf_UDP->id), reportstruct->packetID, reportstruct->packetID);
@@ -554,12 +558,12 @@ inline bool Server::ReadPacketID (void) {
     return terminate;
 }
 
-void Server::L2_processing (void) {
+void Server::L2_processing () {
 #if defined(HAVE_LINUX_FILTER_H) && defined(HAVE_AF_PACKET)
-    eth_hdr = (struct ether_header *) mBuf;
-    ip_hdr = (struct iphdr *) (mBuf + sizeof(struct ether_header));
+    eth_hdr = reinterpret_cast<struct ether_header *>(mBuf);
+    ip_hdr = reinterpret_cast<struct iphdr *>(mBuf + sizeof(struct ether_header));
     // L4 offest is set by the listener and depends upon IPv4 or IPv6
-    udp_hdr = (struct udphdr *) (mBuf + mSettings->l4offset);
+    udp_hdr = reinterpret_cast<struct udphdr *>(mBuf + mSettings->l4offset);
     // Read the packet to get the UDP length
     int udplen = ntohs(udp_hdr->len);
     //
@@ -593,7 +597,7 @@ void Server::L2_processing (void) {
 
 // Run the L2 packet through a quintuple check, i.e. proto/ip src/ip dst/src port/src dst
 // and return zero is there is a match, otherwize return nonzero
-int Server::L2_quintuple_filter (void) {
+int Server::L2_quintuple_filter () {
 #if defined(HAVE_LINUX_FILTER_H) && defined(HAVE_AF_PACKET)
 
 #define IPV4SRCOFFSET 12  // the ipv4 source address offset from the l3 pdu
@@ -603,15 +607,15 @@ int Server::L2_quintuple_filter (void) {
     // Note: it's expected the initiating socket has aready "connected"
     // and the sockaddr structs have been populated
     // 2nd Note:  sockaddr structs are in network byte order
-    struct sockaddr *p = (sockaddr *)&mSettings->peer;
-    struct sockaddr *l = (sockaddr *)&mSettings->local;
+    struct sockaddr *p = reinterpret_cast<sockaddr *>(&mSettings->peer);
+    struct sockaddr *l = reinterpret_cast<sockaddr *>(&mSettings->local);
     // make sure sa_family is coherent for both src and dst
     if (!(((l->sa_family == AF_INET) && (p->sa_family == AF_INET)) || ((l->sa_family == AF_INET6) && (p->sa_family == AF_INET6)))) {
 	return -1;
     }
 
     // check the L2 ethertype
-    struct ether_header *l2hdr = (struct ether_header *)mBuf;
+    struct ether_header *l2hdr = reinterpret_cast<struct ether_header *>(mBuf);
 
     if (!isIPV6(mSettings)) {
 	if (ntohs(l2hdr->ether_type) != ETHERTYPE_IP)
@@ -622,18 +626,18 @@ int Server::L2_quintuple_filter (void) {
     }
     // check the ip src/dst
     const uint32_t *data;
-    udp_hdr = (struct udphdr *) (mBuf + mSettings->l4offset);
+    udp_hdr = reinterpret_cast<struct udphdr *>(mBuf + mSettings->l4offset);
 
     // Check plain old v4 using v4 addr structs
     if (l->sa_family == AF_INET) {
-	data = (const uint32_t *) (mBuf + sizeof(struct ether_header) + IPV4SRCOFFSET);
-	if (((struct sockaddr_in *)(p))->sin_addr.s_addr != *data++)
+	data = reinterpret_cast<const uint32_t *>(mBuf + sizeof(struct ether_header) + IPV4SRCOFFSET);
+	if ((reinterpret_cast<struct sockaddr_in *>(p))->sin_addr.s_addr != *data++)
 	    return -1;
-	if (((struct sockaddr_in *)(l))->sin_addr.s_addr != *data)
+	if ((reinterpret_cast<struct sockaddr_in *>(l))->sin_addr.s_addr != *data)
 	    return -1;
-	if (udp_hdr->source != ((struct sockaddr_in *)(p))->sin_port)
+	if (udp_hdr->source != (reinterpret_cast<struct sockaddr_in *>(p))->sin_port)
 	    return -1;
-	if (udp_hdr->dest != ((struct sockaddr_in *)(l))->sin_port)
+	if (udp_hdr->dest != (reinterpret_cast<struct sockaddr_in *>(l))->sin_port)
 	    return -1;
     } else {
 	// Using the v6 addr structures
@@ -642,7 +646,7 @@ int Server::L2_quintuple_filter (void) {
 	struct in6_addr *v6local = SockAddr_get_in6_addr(&mSettings->local);
 	if (isIPV6(mSettings)) {
 	    int i;
-	    data = (const uint32_t *) (mBuf + sizeof(struct ether_header) + IPV6SRCOFFSET);
+	    data = reinterpret_cast<const uint32_t *>(mBuf + sizeof(struct ether_header) + IPV6SRCOFFSET);
 	    // check for v6 src/dst address match
 	    for (i = 0; i < 4; i++) {
 		if (v6peer->s6_addr32[i] != *data++)
@@ -653,16 +657,16 @@ int Server::L2_quintuple_filter (void) {
 		    return -1;
 	    }
 	} else { // v4 addr in v6 family struct
-	    data = (const uint32_t *) (mBuf + sizeof(struct ether_header) + IPV4SRCOFFSET);
+	    data = reinterpret_cast<const uint32_t *>(mBuf + sizeof(struct ether_header) + IPV4SRCOFFSET);
 	    if (v6peer->s6_addr32[3] != *data++)
 		return -1;
 	    if (v6peer->s6_addr32[3] != *data)
 		return -1;
 	}
 	// check udp ports
-	if (udp_hdr->source != ((struct sockaddr_in6 *)(p))->sin6_port)
+	if (udp_hdr->source != (reinterpret_cast<struct sockaddr_in6 *>(p))->sin6_port)
 	    return -1;
-	if (udp_hdr->dest != ((struct sockaddr_in6 *)(l))->sin6_port)
+	if (udp_hdr->dest != (reinterpret_cast<struct sockaddr_in6 *>(l))->sin6_port)
 	    return -1;
 #  endif // HAVE_IPV6
     }
@@ -673,12 +677,12 @@ int Server::L2_quintuple_filter (void) {
 
 inline void Server::udp_isoch_processing (int rxlen) {
     // Ignore runt sized isoch packets
-    if (rxlen < (int) (sizeof(struct UDP_datagram) +  sizeof(struct client_hdr_v1) + sizeof(struct client_hdrext) + sizeof(struct isoch_payload))) {
+    if (rxlen < static_cast<int>(sizeof(struct UDP_datagram) +  sizeof(struct client_hdr_v1) + sizeof(struct client_hdrext) + sizeof(struct isoch_payload))) {
 	reportstruct->burstsize = 0;
 	reportstruct->remaining = 0;
 	reportstruct->frameID = 0;
     } else {
-	struct client_udp_testhdr *udp_pkt = (struct client_udp_testhdr *)mBuf;
+	struct client_udp_testhdr *udp_pkt = reinterpret_cast<struct client_udp_testhdr *>(mBuf);
 	reportstruct->isochStartTime.tv_sec = ntohl(udp_pkt->isoch.start_tv_sec);
 	reportstruct->isochStartTime.tv_usec = ntohl(udp_pkt->isoch.start_tv_usec);
 	reportstruct->frameID = ntohl(udp_pkt->isoch.frameid);
@@ -686,6 +690,9 @@ inline void Server::udp_isoch_processing (int rxlen) {
 	reportstruct->burstsize = ntohl(udp_pkt->isoch.burstsize);
 	reportstruct->burstperiod = ntohl(udp_pkt->isoch.burstperiod);
 	reportstruct->remaining = ntohl(udp_pkt->isoch.remaining);
+	if ((reportstruct->remaining == rxlen) && ((reportstruct->frameID - reportstruct->prevframeID) == 1)) {
+	    reportstruct->transit_ready = 1;
+	}
     }
 }
 
@@ -694,14 +701,13 @@ inline void Server::udp_isoch_processing (int rxlen) {
  * Sends termination flag several times at the end.
  * Does not close the socket.
  * ------------------------------------------------------------------- */
-void Server::RunUDP (void) {
+void Server::RunUDP () {
     int rxlen;
     int readerr = 0;
-    bool lastpacket = 0;
+    bool lastpacket = false;
 
     if (!InitTrafficLoop())
 	return;
-    struct timeval prevsend = myReport->info.ts.startTime;
 
     // Exit loop on three conditions
     // 1) Fatal read error
@@ -732,21 +738,22 @@ void Server::RunUDP (void) {
 	    }
 	    if (!(reportstruct->l2errors & L2UNKNOWN)) {
 		// ReadPacketID returns true if this is the last UDP packet sent by the client
-		// aslo sets the packet rx time in the reportstruct
-		reportstruct->prevSentTime = prevsend;
+		// also sets the packet rx time in the reportstruct
+		reportstruct->prevSentTime = myReport->info.ts.prevsendTime;
+		reportstruct->prevPacketTime = myReport->info.ts.prevpacketTime;
 		lastpacket = ReadPacketID();
-		prevsend = reportstruct->sentTime;
+		myReport->info.ts.prevsendTime = reportstruct->sentTime;
+		myReport->info.ts.prevpacketTime = reportstruct->packetTime;
 		if (isIsochronous(mSettings)) {
 		    udp_isoch_processing(rxlen);
 		}
 	    }
 	}
-	if (!isSingleUDP(mSettings))
-	    ReportPacket(myReport, reportstruct);
-	else {
-	    packetring_enqueue(myReport->packetring, reportstruct);
-	    reporter_process_transfer_report(myReport);
-	}
+#ifdef HAVE_STRUCT_TCP_INFO_TCPI_TOTAL_RETRANS
+	ReportPacket(myReport, reportstruct, NULL);
+#else
+	ReportPacket(myReport, reportstruct);
+#endif
     }
     disarm_itimer();
     int do_close = EndJob(myJob, reportstruct);
@@ -764,7 +771,7 @@ void Server::RunUDP (void) {
 	int rc = close(mySocket);
 	WARN_errno(rc == SOCKET_ERROR, "server close");
     }
-    Iperf_remove_host(&mSettings->peer);
+    Iperf_remove_host(mSettings);
     FreeReport(myJob);
 }
 // end Recv
