@@ -69,6 +69,7 @@
 #include "Listener.hpp"
 #include "active_hosts.h"
 #include "util.h"
+#include "Reporter.h"
 
 #ifdef WIN32
 #include "service.h"
@@ -78,7 +79,9 @@
  * prototypes
  * ------------------------------------------------------------------- */
 // Function called at exit to clean up as much as possible
-void cleanup(void);
+void cleanup();
+// signal handlers
+static void Sig_Interupt(int inSigno);
 
 /* -------------------------------------------------------------------
  * global variables
@@ -90,7 +93,7 @@ extern "C" {
     // as identifier for SUM reports
     int groupID = 0;
     // Mutex to protect access to the above ID
-    Mutex groupCond;
+    Mutex transferid_mutex;
     // Condition used to signal the reporter thread
     // when a packet ring is full.  Shouldn't really
     // be needed but is "belts and suspeners"
@@ -111,7 +114,7 @@ nthread_t sThread;
 static thread_Settings* ext_gSettings;
 // The main thread uses this function to wait
 // for all other threads to complete
-void waitUntilQuit(void);
+void waitUntilQuit();
 
 /* -------------------------------------------------------------------
  * main()
@@ -130,7 +133,6 @@ int main(int argc, char **argv) {
     my_signal(SIGINT,  Sig_Interupt);
 #ifndef WIN32
     my_signal(SIGALRM,  Sig_Interupt);
-
     // Ignore broken pipes
     signal(SIGPIPE,SIG_IGN);
 #else
@@ -140,7 +142,6 @@ int main(int argc, char **argv) {
     WARN_errno(rc == SOCKET_ERROR, "WSAStartup");
     if (rc == SOCKET_ERROR)
 	return 0;
-
     // Tell windows we want to handle our own signals
     SetConsoleCtrlHandler(sig_dispatcher, true);
 #endif
@@ -153,6 +154,8 @@ int main(int argc, char **argv) {
     Mutex_Initialize(&packetringdebug_mutex);
     Mutex_Initialize(&thread_debug_mutex);
 #endif
+    Mutex_Initialize(&transferid_mutex);
+
     // Initialize reporter thread mutex
     reporter_state.ready = 0;
     threads_start.ready = 0;
@@ -172,7 +175,6 @@ int main(int argc, char **argv) {
 
     // Allocate the "global" settings
     ext_gSettings = new thread_Settings;
-
     // Default reporting mode here to avoid unitialized warnings
     // this won't be the actual mode
     ThreadMode ReporterThreadMode = kMode_Reporter;
@@ -207,6 +209,21 @@ int main(int argc, char **argv) {
 	return 0;
     }
 
+    if (!isSTDOUT(ext_gSettings)) {
+#ifdef HAVE_FREOPEN
+	FILE *fd;
+	fprintf(stdout, "Output from stdout and stderr will be redirected to file %s\n", ext_gSettings->mOutputFileName);
+	fflush(stdout);
+	fd = freopen(ext_gSettings->mOutputFileName, "w", stdout);
+	FAIL_errno(fd == NULL, "freopen stdout\n", ext_gSettings);
+	fd = freopen(ext_gSettings->mOutputFileName, "w", stderr);
+	FAIL_errno(fd == NULL, "freopen stderr\n", ext_gSettings);
+#else
+	fprintf(stderr, "Output to file not supported\n");
+#endif
+
+    }
+
     unsetReport(ext_gSettings);
     switch (ext_gSettings->mThreadMode) {
     case kMode_Client :
@@ -244,6 +261,10 @@ int main(int argc, char **argv) {
 	    }
 	}
 #endif
+	// Start up any parallel listener threads
+	if (ext_gSettings->mPortLast) {
+	    listeners_init(ext_gSettings);
+	}
 	break;
     default :
 	fprintf(stderr, "unknown mode");
@@ -269,10 +290,8 @@ int main(int argc, char **argv) {
     // No need to make a reporter thread because we don't have threads
     thread_start(ext_gSettings);
 #endif
-
     // wait for other (client, server) threads to complete
     thread_joinall();
-
     // all done!
     return 0;
 } // end main
@@ -282,7 +301,7 @@ int main(int argc, char **argv) {
  * respond appropriately.. [static]
  * ------------------------------------------------------------------- */
 
-void Sig_Interupt(int inSigno) {
+void Sig_Interupt (int inSigno) {
 #ifdef HAVE_THREAD
     // We try to not allow a single interrupt handled by multiple threads
     // to completely kill the app so we save off the first thread ID
@@ -314,7 +333,7 @@ void Sig_Interupt(int inSigno) {
  * either by exit() or terminating main().
  * ------------------------------------------------------------------- */
 
-void cleanup(void) {
+void cleanup () {
 #ifdef WIN32
     // Shutdown Winsock
     WSACleanup();
@@ -332,6 +351,7 @@ void cleanup(void) {
     Mutex_Destroy(&packetringdebug_mutex);
     Mutex_Destroy(&thread_debug_mutex);
 #endif
+    Mutex_Destroy(&transferid_mutex);
     // shutdown the thread subsystem
     thread_destroy();
 } // end cleanup
@@ -420,7 +440,6 @@ VOID ServiceStart (DWORD dwArgc, LPTSTR *lpszArgv) {
     thread_joinall();
 }
 
-
 //
 //  FUNCTION: ServiceStop
 //
